# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Literal, Required, TypeAlias, TypedDict

from .._types import SequenceNotStr

__all__ = [
    "EvalCreateParams",
    "Parameters",
    "ParametersEvaluationClassifyParameters",
    "ParametersEvaluationClassifyParametersJudge",
    "ParametersEvaluationClassifyParametersModelToEvaluate",
    "ParametersEvaluationClassifyParametersModelToEvaluateEvaluationModelRequest",
    "ParametersEvaluationScoreParameters",
    "ParametersEvaluationScoreParametersJudge",
    "ParametersEvaluationScoreParametersModelToEvaluate",
    "ParametersEvaluationScoreParametersModelToEvaluateEvaluationModelRequest",
    "ParametersEvaluationCompareParameters",
    "ParametersEvaluationCompareParametersJudge",
    "ParametersEvaluationCompareParametersModelA",
    "ParametersEvaluationCompareParametersModelAEvaluationModelRequest",
    "ParametersEvaluationCompareParametersModelB",
    "ParametersEvaluationCompareParametersModelBEvaluationModelRequest",
]


class EvalCreateParams(TypedDict, total=False):
    parameters: Required[Parameters]
    """Type-specific parameters for the evaluation"""

    type: Required[Literal["classify", "score", "compare"]]
    """The type of evaluation to perform"""


class ParametersEvaluationClassifyParametersJudge(TypedDict, total=False):
    model: Required[str]
    """Name of the judge model"""

    model_source: Required[Literal["serverless", "dedicated", "external"]]
    """Source of the judge model."""

    system_template: Required[str]
    """System prompt template for the judge"""

    external_api_token: str
    """Bearer/API token for external judge models."""

    external_base_url: str
    """Base URL for external judge models. Must be OpenAI-compatible base URL."""


class ParametersEvaluationClassifyParametersModelToEvaluateEvaluationModelRequest(TypedDict, total=False):
    input_template: Required[str]
    """Input prompt template"""

    max_tokens: Required[int]
    """Maximum number of tokens to generate"""

    model: Required[str]
    """Name of the model to evaluate"""

    model_source: Required[Literal["serverless", "dedicated", "external"]]
    """Source of the model."""

    system_template: Required[str]
    """System prompt template"""

    temperature: Required[float]
    """Sampling temperature"""

    external_api_token: str
    """Bearer/API token for external models."""

    external_base_url: str
    """Base URL for external models. Must be OpenAI-compatible base URL"""


ParametersEvaluationClassifyParametersModelToEvaluate: TypeAlias = Union[
    str, ParametersEvaluationClassifyParametersModelToEvaluateEvaluationModelRequest
]


class ParametersEvaluationClassifyParameters(TypedDict, total=False):
    input_data_file_path: Required[str]
    """Data file ID"""

    judge: Required[ParametersEvaluationClassifyParametersJudge]

    labels: Required[SequenceNotStr[str]]
    """List of possible classification labels"""

    pass_labels: Required[SequenceNotStr[str]]
    """List of labels that are considered passing"""

    model_to_evaluate: ParametersEvaluationClassifyParametersModelToEvaluate
    """Field name in the input data"""


class ParametersEvaluationScoreParametersJudge(TypedDict, total=False):
    model: Required[str]
    """Name of the judge model"""

    model_source: Required[Literal["serverless", "dedicated", "external"]]
    """Source of the judge model."""

    system_template: Required[str]
    """System prompt template for the judge"""

    external_api_token: str
    """Bearer/API token for external judge models."""

    external_base_url: str
    """Base URL for external judge models. Must be OpenAI-compatible base URL."""


class ParametersEvaluationScoreParametersModelToEvaluateEvaluationModelRequest(TypedDict, total=False):
    input_template: Required[str]
    """Input prompt template"""

    max_tokens: Required[int]
    """Maximum number of tokens to generate"""

    model: Required[str]
    """Name of the model to evaluate"""

    model_source: Required[Literal["serverless", "dedicated", "external"]]
    """Source of the model."""

    system_template: Required[str]
    """System prompt template"""

    temperature: Required[float]
    """Sampling temperature"""

    external_api_token: str
    """Bearer/API token for external models."""

    external_base_url: str
    """Base URL for external models. Must be OpenAI-compatible base URL"""


ParametersEvaluationScoreParametersModelToEvaluate: TypeAlias = Union[
    str, ParametersEvaluationScoreParametersModelToEvaluateEvaluationModelRequest
]


class ParametersEvaluationScoreParameters(TypedDict, total=False):
    input_data_file_path: Required[str]
    """Data file ID"""

    judge: Required[ParametersEvaluationScoreParametersJudge]

    max_score: Required[float]
    """Maximum possible score"""

    min_score: Required[float]
    """Minimum possible score"""

    pass_threshold: Required[float]
    """Score threshold for passing"""

    model_to_evaluate: ParametersEvaluationScoreParametersModelToEvaluate
    """Field name in the input data"""


class ParametersEvaluationCompareParametersJudge(TypedDict, total=False):
    model: Required[str]
    """Name of the judge model"""

    model_source: Required[Literal["serverless", "dedicated", "external"]]
    """Source of the judge model."""

    system_template: Required[str]
    """System prompt template for the judge"""

    external_api_token: str
    """Bearer/API token for external judge models."""

    external_base_url: str
    """Base URL for external judge models. Must be OpenAI-compatible base URL."""


class ParametersEvaluationCompareParametersModelAEvaluationModelRequest(TypedDict, total=False):
    input_template: Required[str]
    """Input prompt template"""

    max_tokens: Required[int]
    """Maximum number of tokens to generate"""

    model: Required[str]
    """Name of the model to evaluate"""

    model_source: Required[Literal["serverless", "dedicated", "external"]]
    """Source of the model."""

    system_template: Required[str]
    """System prompt template"""

    temperature: Required[float]
    """Sampling temperature"""

    external_api_token: str
    """Bearer/API token for external models."""

    external_base_url: str
    """Base URL for external models. Must be OpenAI-compatible base URL"""


ParametersEvaluationCompareParametersModelA: TypeAlias = Union[
    str, ParametersEvaluationCompareParametersModelAEvaluationModelRequest
]


class ParametersEvaluationCompareParametersModelBEvaluationModelRequest(TypedDict, total=False):
    input_template: Required[str]
    """Input prompt template"""

    max_tokens: Required[int]
    """Maximum number of tokens to generate"""

    model: Required[str]
    """Name of the model to evaluate"""

    model_source: Required[Literal["serverless", "dedicated", "external"]]
    """Source of the model."""

    system_template: Required[str]
    """System prompt template"""

    temperature: Required[float]
    """Sampling temperature"""

    external_api_token: str
    """Bearer/API token for external models."""

    external_base_url: str
    """Base URL for external models. Must be OpenAI-compatible base URL"""


ParametersEvaluationCompareParametersModelB: TypeAlias = Union[
    str, ParametersEvaluationCompareParametersModelBEvaluationModelRequest
]


class ParametersEvaluationCompareParameters(TypedDict, total=False):
    input_data_file_path: Required[str]
    """Data file name"""

    judge: Required[ParametersEvaluationCompareParametersJudge]

    model_a: ParametersEvaluationCompareParametersModelA
    """Field name in the input data"""

    model_b: ParametersEvaluationCompareParametersModelB
    """Field name in the input data"""


Parameters: TypeAlias = Union[
    ParametersEvaluationClassifyParameters, ParametersEvaluationScoreParameters, ParametersEvaluationCompareParameters
]



from __future__ import annotations

import os
from typing import Any

import click

import together
from together._version import __version__
from together._constants import DEFAULT_TIMEOUT
from together.lib.cli.api.evals import evals
from together.lib.cli.api.files import files
from together.lib.cli.api.models import models
from together.lib.cli.api.endpoints import endpoints
from together.lib.cli.api.fine_tuning import fine_tuning


def print_version(ctx: click.Context, _params: Any, value: Any) -> None:
    if not value or ctx.resilient_parsing:
        return
    click.echo(f"Version {__version__}")
    ctx.exit()


@click.group()
@click.pass_context
@click.option(
    "--api-key",
    type=str,
    help="API Key. Defaults to environment variable `TOGETHER_API_KEY`",
    default=os.getenv("TOGETHER_API_KEY"),
)
@click.option("--base-url", type=str, help="API Base URL. Defaults to Together AI endpoint.")
@click.option("--timeout", type=int, help=f"Request timeout. Defaults to {DEFAULT_TIMEOUT} seconds")
@click.option(
    "--max-retries",
    type=int,
    help=f"Maximum number of HTTP retries.",
)
@click.option(
    "--version",
    is_flag=True,
    callback=print_version,
    expose_value=False,
    is_eager=True,
    help="Print version",
)
@click.option("--debug", help="Debug mode", is_flag=True)
def main(
    ctx: click.Context,
    api_key: str | None,
    base_url: str | None,
    timeout: int | None,
    debug: bool | None,
    max_retries: int | None,
) -> None:
    """This is a sample CLI tool."""
    os.environ.setdefault("TOGETHER_LOG", "debug" if debug else "info")
    ctx.obj = together.Together(
        api_key=api_key, base_url=base_url, timeout=timeout, max_retries=max_retries if max_retries is not None else 0
    )


main.add_command(files)
main.add_command(fine_tuning)
main.add_command(models)
main.add_command(endpoints)
main.add_command(evals)

if __name__ == "__main__":
    main()



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["TrainingMethodDpo"]


class TrainingMethodDpo(BaseModel):
    method: Literal["dpo"] = "dpo"

    dpo_beta: Optional[float] = None

    dpo_normalize_logratios_by_length: Optional[bool] = None

    dpo_reference_free: Optional[bool] = None

    rpo_alpha: Optional[float] = None

    simpo_gamma: Optional[float] = None



import sys
import json
from typing import Any, Dict, List, Union, Literal, TypeVar, Callable, Optional, cast
from functools import wraps

import click
from tabulate import tabulate

from together import APIError, Together, TogetherError
from together._types import omit
from together.lib.utils.serializer import datetime_serializer
from together.types.eval_create_params import (
    ParametersEvaluationScoreParameters,
    ParametersEvaluationCompareParameters,
    ParametersEvaluationClassifyParameters,
    ParametersEvaluationScoreParametersJudge,
    ParametersEvaluationCompareParametersJudge,
    ParametersEvaluationClassifyParametersJudge,
    ParametersEvaluationScoreParametersModelToEvaluate,
    ParametersEvaluationClassifyParametersModelToEvaluate,
    ParametersEvaluationCompareParametersModelAEvaluationModelRequest,
    ParametersEvaluationCompareParametersModelBEvaluationModelRequest,
)


def print_api_error(e: Union[APIError, TogetherError]) -> None:
    if isinstance(e, APIError):
        error_details = cast(Dict[str, Any], e.body)["error"]["message"]

        if error_details and ("credentials" in error_details.lower() or "authentication" in error_details.lower()):
            click.echo("Error: Invalid API key or authentication failed", err=True)
        else:
            click.echo(f"Error: {error_details}", err=True)

    click.echo(f"Error: {e}", err=True)
    return


F = TypeVar("F", bound=Callable[..., Any])


def handle_api_errors(f: F) -> F:
    """Decorator to handle common API errors in CLI commands."""

    @wraps(f)
    def wrapper(*args: Any, **kwargs: Any) -> Any:
        try:
            return f(*args, **kwargs)
        except APIError as e:
            print_api_error(e)
            sys.exit(1)
        except TogetherError as e:
            print_api_error(e)
            sys.exit(1)
        except Exception as e:
            click.echo(f"Error: An unexpected error occurred - {str(e)}", err=True)
            sys.exit(1)

    return wrapper  # type: ignore


@click.group()
@click.pass_context
def evals(ctx: click.Context) -> None:
    """Evals API commands"""
    pass


@evals.command()
@click.pass_context
@click.option(
    "--type",
    type=click.Choice(["classify", "score", "compare"]),
    required=True,
    help="Type of evaluation to create.",
)
@click.option(
    "--judge-model",
    type=str,
    required=True,
    help="Name or URL of the judge model to use for evaluation.",
)
@click.option(
    "--judge-model-source",
    type=click.Choice(["serverless", "dedicated", "external"]),
    required=True,
    help="Source of the judge model.",
)
@click.option(
    "--judge-external-api-token",
    type=str,
    required=False,
    help="Optional external API token for the judge model.",
)
@click.option(
    "--judge-external-base-url",
    type=str,
    required=False,
    help="Optional external base URLs for the judge model.",
)
@click.option(
    "--judge-system-template",
    type=str,
    required=True,
    help="System template for the judge model.",
)
@click.option(
    "--input-data-file-path",
    type=str,
    required=True,
    help="Path to the input data file.",
)
@click.option(
    "--model-field",
    type=str,
    help="Name of the field in the input file contaning text generated by the model."
    "Can not be used when model-a-name and other model config parameters are specified",
)
@click.option(
    "--model-to-evaluate",
    type=str,
    help="Model name when using the detailed config",
)
@click.option(
    "--model-to-evaluate-source",
    type=click.Choice(["serverless", "dedicated", "external"]),
    help="Source of the model to evaluate.",
)
@click.option(
    "--model-to-evaluate-external-api-token",
    type=str,
    help="Optional external API token for the model to evaluate.",
)
@click.option(
    "--model-to-evaluate-external-base-url",
    type=str,
    help="Optional external base URL for the model to evaluate.",
)
@click.option(
    "--model-to-evaluate-max-tokens",
    type=int,
    help="Max tokens for model-to-evaluate",
)
@click.option(
    "--model-to-evaluate-temperature",
    type=float,
    help="Temperature for model-to-evaluate",
)
@click.option(
    "--model-to-evaluate-system-template",
    type=str,
    help="System template for model-to-evaluate",
)
@click.option(
    "--model-to-evaluate-input-template",
    type=str,
    help="Input template for model-to-evaluate",
)
@click.option(
    "--labels",
    type=str,
    help="Classification labels - comma-separated list",
)
@click.option(
    "--pass-labels",
    type=str,
    help="Labels considered as passing (required for classify type). A comma-separated list.",
)
@click.option(
    "--min-score",
    type=float,
    help="Minimum score value (required for score type).",
)
@click.option(
    "--max-score",
    type=float,
    help="Maximum score value (required for score type).",
)
@click.option(
    "--pass-threshold",
    type=float,
    help="Threshold score for passing (required for score type).",
)
@click.option(
    "--model-a-field",
    type=str,
    help="Name of the field in the input file containing text generated by Model A. \
        Can not be used when model-a-name and other model config parameters are specified",
)
@click.option(
    "--model-a",
    type=str,
    help="Model name or URL for model A when using detailed config.",
)
@click.option(
    "--model-a-source",
    type=click.Choice(["serverless", "dedicated", "external"]),
    help="Source of model A.",
)
@click.option(
    "--model-a-external-api-token",
    type=str,
    help="Optional external API token for model A.",
)
@click.option(
    "--model-a-external-base-url",
    type=str,
    help="Optional external base URL for model A.",
)
@click.option(
    "--model-a-max-tokens",
    type=int,
    help="Max tokens for model A.",
)
@click.option(
    "--model-a-temperature",
    type=float,
    help="Temperature for model A.",
)
@click.option(
    "--model-a-system-template",
    type=str,
    help="System template for model A.",
)
@click.option(
    "--model-a-input-template",
    type=str,
    help="Input template for model A.",
)
@click.option(
    "--model-b-field",
    type=str,
    help="Name of the field in the input file containing text generated by Model B.\
          Can not be used when model-b-name and other model config parameters are specified",
)
@click.option(
    "--model-b",
    type=str,
    help="Model name or URL for model B when using detailed config.",
)
@click.option(
    "--model-b-source",
    type=click.Choice(["serverless", "dedicated", "external"]),
    help="Source of model B.",
)
@click.option(
    "--model-b-external-api-token",
    type=str,
    help="Optional external API token for model B.",
)
@click.option(
    "--model-b-external-base-url",
    type=str,
    help="Optional external base URL for model B.",
)
@click.option(
    "--model-b-max-tokens",
    type=int,
    help="Max tokens for model B.",
)
@click.option(
    "--model-b-temperature",
    type=float,
    help="Temperature for model B.",
)
@click.option(
    "--model-b-system-template",
    type=str,
    help="System template for model B.",
)
@click.option(
    "--model-b-input-template",
    type=str,
    help="Input template for model B.",
)
@handle_api_errors
def create(
    ctx: click.Context,
    type: Literal["classify", "score", "compare"],
    judge_model: str,
    judge_model_source: Literal["serverless", "dedicated", "external"],
    judge_system_template: str,
    judge_external_api_token: Optional[str],
    judge_external_base_url: Optional[str],
    input_data_file_path: str,
    model_field: Optional[str],
    model_to_evaluate: Optional[str],
    model_to_evaluate_source: Optional[str],
    model_to_evaluate_external_api_token: Optional[str],
    model_to_evaluate_external_base_url: Optional[str],
    model_to_evaluate_max_tokens: Optional[int],
    model_to_evaluate_temperature: Optional[float],
    model_to_evaluate_system_template: Optional[str],
    model_to_evaluate_input_template: Optional[str],
    labels: str,
    pass_labels: str,
    min_score: Optional[float],
    max_score: Optional[float],
    pass_threshold: Optional[float],
    model_a_field: Optional[str],
    model_a: Optional[str],
    model_a_source: Optional[str],
    model_a_external_api_token: Optional[str],
    model_a_external_base_url: Optional[str],
    model_a_max_tokens: Optional[int],
    model_a_temperature: Optional[float],
    model_a_system_template: Optional[str],
    model_a_input_template: Optional[str],
    model_b_field: Optional[str],
    model_b: Optional[str],
    model_b_source: Optional[str],
    model_b_external_api_token: Optional[str],
    model_b_external_base_url: Optional[str],
    model_b_max_tokens: Optional[int],
    model_b_temperature: Optional[float],
    model_b_system_template: Optional[str],
    model_b_input_template: Optional[str],
) -> None:
    """Create a new evaluation job"""

    client: Together = ctx.obj

    # Convert strings to lists for labels
    labels_list = labels.split(",") if labels else None
    pass_labels_list = pass_labels.split(",") if pass_labels else None

    # Build model configurations
    model_to_evaluate_final: Union[Dict[str, Any], None, str] = None

    # Check if any config parameters are provided
    config_params_provided = any(
        [
            model_to_evaluate,
            model_to_evaluate_source,
            model_to_evaluate_max_tokens,
            model_to_evaluate_temperature,
            model_to_evaluate_system_template,
            model_to_evaluate_input_template,
        ]
    )

    if model_field:
        # Simple mode: model_field is provided
        if config_params_provided:
            raise click.BadParameter(
                "Cannot specify both --model-field and --model-to-evaluate-* parameters. "
                "Use either --model-field alone if your input file has pre-generated responses, "
                "or config parameters if you want to generate it on our end"
            )
        model_to_evaluate_final = model_field
    elif config_params_provided:
        # Config mode: config parameters are provided
        model_to_evaluate_final = {
            "model": model_to_evaluate,
            "model_source": model_to_evaluate_source,
            "max_tokens": model_to_evaluate_max_tokens,
            "temperature": model_to_evaluate_temperature,
            "system_template": model_to_evaluate_system_template,
            "input_template": model_to_evaluate_input_template,
        }
        if model_to_evaluate_external_api_token:
            model_to_evaluate_final["external_api_token"] = model_to_evaluate_external_api_token
        if model_to_evaluate_external_base_url:
            model_to_evaluate_final["external_base_url"] = model_to_evaluate_external_base_url

    # Build model-a configuration
    model_a_final: Union[Dict[str, Any], None, str] = None
    model_a_config_params = [
        model_a,
        model_a_source,
        model_a_max_tokens,
        model_a_temperature,
        model_a_system_template,
        model_a_input_template,
    ]

    if model_a_field is not None:
        # Simple mode: model_a_field is provided
        if any(model_a_config_params):
            raise click.BadParameter(
                "Cannot specify both --model-a-field and config parameters (--model-a-name, etc.). "
                "Use either --model-a-field alone if your input file has pre-generated responses, "
                "or config parameters if you want to generate it on our end"
            )
        model_a_final = model_a_field
    elif any(model_a_config_params):
        # Config mode: config parameters are provided
        model_a_final = {
            "model": model_a,
            "model_source": model_a_source,
            "max_tokens": model_a_max_tokens,
            "temperature": model_a_temperature,
            "system_template": model_a_system_template,
            "input_template": model_a_input_template,
        }
        if model_a_external_api_token:
            model_a_final["external_api_token"] = model_a_external_api_token
        if model_a_external_base_url:
            model_a_final["external_base_url"] = model_a_external_base_url

    # Build model-b configuration
    model_b_final: Union[Dict[str, Any], None, str] = None
    model_b_config_params = [
        model_b,
        model_b_source,
        model_b_max_tokens,
        model_b_temperature,
        model_b_system_template,
        model_b_input_template,
    ]

    if model_b_field is not None:
        # Simple mode: model_b_field is provided
        if any(model_b_config_params):
            raise click.BadParameter(
                "Cannot specify both --model-b-field and config parameters (--model-b-name, etc.). "
                "Use either --model-b-field alone if your input file has pre-generated responses, "
                "or config parameters if you want to generate it on our end"
            )
        model_b_final = model_b_field
    elif any(model_b_config_params):
        # Config mode: config parameters are provided
        model_b_final = {
            "model": model_b,
            "model_source": model_b_source,
            "max_tokens": model_b_max_tokens,
            "temperature": model_b_temperature,
            "system_template": model_b_system_template,
            "input_template": model_b_input_template,
        }
        if model_b_external_api_token:
            model_b_final["external_api_token"] = model_b_external_api_token
        if model_b_external_base_url:
            model_b_final["external_base_url"] = model_b_external_base_url

    judge_config = _build_judge(
        type, judge_model, judge_model_source, judge_system_template, judge_external_api_token, judge_external_base_url
    )

    if type == "classify":
        response = client.evals.create(
            type=type,
            parameters=ParametersEvaluationClassifyParameters(
                input_data_file_path=input_data_file_path,
                judge=judge_config,
                labels=labels_list or [],
                pass_labels=pass_labels_list or [],
                model_to_evaluate=cast(ParametersEvaluationClassifyParametersModelToEvaluate, model_to_evaluate_final),
            ),
        )
    elif type == "score":
        if max_score is None or min_score is None or pass_threshold is None:
            raise TogetherError("max_score, min_score, and pass_threshold are required for score type")

        response = client.evals.create(
            type="score",
            parameters=ParametersEvaluationScoreParameters(
                input_data_file_path=input_data_file_path,
                judge=judge_config,
                max_score=max_score,
                min_score=min_score,
                pass_threshold=pass_threshold,
                model_to_evaluate=cast(ParametersEvaluationScoreParametersModelToEvaluate, model_to_evaluate_final),
            ),
        )
    elif type == "compare":
        response = client.evals.create(
            type=type,
            parameters=ParametersEvaluationCompareParameters(
                input_data_file_path=input_data_file_path,
                judge=judge_config,
                model_a=cast(ParametersEvaluationCompareParametersModelAEvaluationModelRequest, model_a_final),
                model_b=cast(ParametersEvaluationCompareParametersModelBEvaluationModelRequest, model_b_final),
            ),
        )

    click.echo(json.dumps(response.model_dump(exclude_none=True), indent=4))


@evals.command()
@click.pass_context
@click.option(
    "--status",
    type=click.Choice(["pending", "queued", "running", "completed", "error", "user_error"]),
    help="Filter by job status.",
)
@click.option(
    "--limit",
    type=int,
    help="Limit number of results (max 100).",
)
def list(
    ctx: click.Context,
    status: Union[Literal["pending", "queued", "running", "completed", "error", "user_error"], None],
    limit: Union[int, None],
) -> None:
    """List evals"""

    client: Together = ctx.obj

    response = client.evals.list(status=status or omit, limit=limit or omit)

    display_list: List[Dict[str, Any]] = []
    for job in response:
        if job.parameters:
            model = job.parameters.get("model_to_evaluate", "")
            model_a = job.parameters.get("model_a", "")
            model_b = job.parameters.get("model_b", "")
        else:
            model = ""
            model_a = ""
            model_b = ""

        display_list.append(
            {
                "Workflow ID": job.workflow_id or "",
                "Type": job.type,
                "Status": job.status,
                "Created At": job.created_at or 0,
                "Model": model,
                "Model A": model_a,
                "Model B": model_b,
            }
        )

    table = tabulate(display_list, headers="keys", tablefmt="grid", showindex=True)
    click.echo(table)


@evals.command()
@click.pass_context
@click.argument("evaluation_id", type=str, required=True)
def retrieve(ctx: click.Context, evaluation_id: str) -> None:
    """Get details of a specific evaluation job"""

    client: Together = ctx.obj

    response = client.evals.retrieve(evaluation_id)

    click.echo(json.dumps(response.model_dump(exclude_none=True), default=datetime_serializer, indent=4))


@evals.command()
@click.pass_context
@click.argument("evaluation_id", type=str, required=True)
def status(ctx: click.Context, evaluation_id: str) -> None:
    """Get the status and results of a specific evaluation job"""

    client: Together = ctx.obj

    response = client.evals.status(evaluation_id)

    click.echo(json.dumps(response.model_dump(exclude_none=True), indent=4))


def _build_judge(
    type: Literal["classify", "score", "compare"],
    judge_model: str,
    judge_model_source: Literal["serverless", "dedicated", "external"],
    judge_system_template: str,
    judge_external_api_token: Optional[str],
    judge_external_base_url: Optional[str],
) -> ParametersEvaluationClassifyParametersJudge:
    if type == "classify":
        judge_config = ParametersEvaluationClassifyParametersJudge(
            model=judge_model,
            model_source=judge_model_source,
            system_template=judge_system_template,
        )
    elif type == "score":
        judge_config = ParametersEvaluationScoreParametersJudge(
            model=judge_model,
            model_source=judge_model_source,
            system_template=judge_system_template,
        )
    elif type == "compare":
        judge_config = ParametersEvaluationCompareParametersJudge(
            model=judge_model,
            model_source=judge_model_source,
            system_template=judge_system_template,
        )

    if judge_external_api_token:
        judge_config["external_api_token"] = judge_external_api_token
    if judge_external_base_url:
        judge_config["external_base_url"] = judge_external_base_url

    return judge_config



import os
import json
from pathlib import Path

import pytest

from together import Together
from together.types import (
    FileResponse,
)


class TestTogetherFiles:
    @pytest.fixture
    def sync_together_client(self) -> Together:
        """
        Initialize object with mocked API key
        """
        TOGETHER_API_KEY = os.getenv("TOGETHER_API_KEY")
        return Together(api_key=TOGETHER_API_KEY)

    def test_file_upload(
        self,
        sync_together_client: Together,
        tmp_path: Path,
    ):
        # Mock the post method directly on the client
        files = sync_together_client.files

        # Make a temporary file object
        file = tmp_path / "valid.jsonl"
        content = [{"text": "Hello, world!"}, {"text": "How are you?"}]
        with file.open("w") as f:
            f.write("\n".join(json.dumps(item) for item in content))

        # Test run method
        response = files.upload(
            file,
        )

        # Verify the response
        assert isinstance(response, FileResponse)
        assert response.filename == "valid.jsonl"
        assert response.file_type == "jsonl"
        assert response.line_count == 0
        assert response.object == "file"
        assert response.processed == True
        assert response.purpose == "fine-tune"



from .error import (
    DownloadError,
    FileTypeError,
)
from .fine_tuning import (
    FinetuneTrainingLimits,
)

__all__ = [
    "FinetuneTrainingLimits",
    "DownloadError",
    "FileTypeError",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Any, List, Union, Mapping, cast
from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, FileTypes, omit, not_given
from ..._utils import extract_files, maybe_transform, deepcopy_minimal, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...types.audio import translation_create_params
from ..._base_client import make_request_options
from ...types.audio.translation_create_response import TranslationCreateResponse

__all__ = ["TranslationsResource", "AsyncTranslationsResource"]


class TranslationsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> TranslationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/togethercomputer/together-py#accessing-raw-response-data-eg-headers
        """
        return TranslationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TranslationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/togethercomputer/together-py#with_streaming_response
        """
        return TranslationsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        file: FileTypes,
        language: str | Omit = omit,
        model: Literal["openai/whisper-large-v3"] | Omit = omit,
        prompt: str | Omit = omit,
        response_format: Literal["json", "verbose_json"] | Omit = omit,
        temperature: float | Omit = omit,
        timestamp_granularities: Union[Literal["segment", "word"], List[Literal["segment", "word"]]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TranslationCreateResponse:
        """
        Translates audio into English

        Args:
          file: Audio file to translate

          language: Target output language. Optional ISO 639-1 language code. If omitted, language
              is set to English.

          model: Model to use for translation

          prompt: Optional text to bias decoding.

          response_format: The format of the response

          temperature: Sampling temperature between 0.0 and 1.0

          timestamp_granularities: Controls level of timestamp detail in verbose_json. Only used when
              response_format is verbose_json. Can be a single granularity or an array to get
              multiple levels.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        body = deepcopy_minimal(
            {
                "file": file,
                "language": language,
                "model": model,
                "prompt": prompt,
                "response_format": response_format,
                "temperature": temperature,
                "timestamp_granularities": timestamp_granularities,
            }
        )
        files = extract_files(cast(Mapping[str, object], body), paths=[["file"]])
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return cast(
            TranslationCreateResponse,
            self._post(
                "/audio/translations",
                body=maybe_transform(body, translation_create_params.TranslationCreateParams),
                files=files,
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(
                    Any, TranslationCreateResponse
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )


class AsyncTranslationsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncTranslationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/togethercomputer/together-py#accessing-raw-response-data-eg-headers
        """
        return AsyncTranslationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTranslationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/togethercomputer/together-py#with_streaming_response
        """
        return AsyncTranslationsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        file: FileTypes,
        language: str | Omit = omit,
        model: Literal["openai/whisper-large-v3"] | Omit = omit,
        prompt: str | Omit = omit,
        response_format: Literal["json", "verbose_json"] | Omit = omit,
        temperature: float | Omit = omit,
        timestamp_granularities: Union[Literal["segment", "word"], List[Literal["segment", "word"]]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TranslationCreateResponse:
        """
        Translates audio into English

        Args:
          file: Audio file to translate

          language: Target output language. Optional ISO 639-1 language code. If omitted, language
              is set to English.

          model: Model to use for translation

          prompt: Optional text to bias decoding.

          response_format: The format of the response

          temperature: Sampling temperature between 0.0 and 1.0

          timestamp_granularities: Controls level of timestamp detail in verbose_json. Only used when
              response_format is verbose_json. Can be a single granularity or an array to get
              multiple levels.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        body = deepcopy_minimal(
            {
                "file": file,
                "language": language,
                "model": model,
                "prompt": prompt,
                "response_format": response_format,
                "temperature": temperature,
                "timestamp_granularities": timestamp_granularities,
            }
        )
        files = extract_files(cast(Mapping[str, object], body), paths=[["file"]])
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return cast(
            TranslationCreateResponse,
            await self._post(
                "/audio/translations",
                body=await async_maybe_transform(body, translation_create_params.TranslationCreateParams),
                files=files,
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(
                    Any, TranslationCreateResponse
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )


class TranslationsResourceWithRawResponse:
    def __init__(self, translations: TranslationsResource) -> None:
        self._translations = translations

        self.create = to_raw_response_wrapper(
            translations.create,
        )


class AsyncTranslationsResourceWithRawResponse:
    def __init__(self, translations: AsyncTranslationsResource) -> None:
        self._translations = translations

        self.create = async_to_raw_response_wrapper(
            translations.create,
        )


class TranslationsResourceWithStreamingResponse:
    def __init__(self, translations: TranslationsResource) -> None:
        self._translations = translations

        self.create = to_streamed_response_wrapper(
            translations.create,
        )


class AsyncTranslationsResourceWithStreamingResponse:
    def __init__(self, translations: AsyncTranslationsResource) -> None:
        self._translations = translations

        self.create = async_to_streamed_response_wrapper(
            translations.create,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from .._models import BaseModel
from .finetune_event import FinetuneEvent

__all__ = ["FineTuningListEventsResponse"]


class FineTuningListEventsResponse(BaseModel):
    data: List[FinetuneEvent]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import batch_create_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.batch_job import BatchJob
from ..types.batch_list_response import BatchListResponse
from ..types.batch_create_response import BatchCreateResponse

__all__ = ["BatchesResource", "AsyncBatchesResource"]


class BatchesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> BatchesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/togethercomputer/together-py#accessing-raw-response-data-eg-headers
        """
        return BatchesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> BatchesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/togethercomputer/together-py#with_streaming_response
        """
        return BatchesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        endpoint: str,
        input_file_id: str,
        completion_window: str | Omit = omit,
        model_id: str | Omit = omit,
        priority: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BatchCreateResponse:
        """
        Create a new batch job with the given input file and endpoint

        Args:
          endpoint: The endpoint to use for batch processing

          input_file_id: ID of the uploaded input file containing batch requests

          completion_window: Time window for batch completion (optional)

          model_id: Model to use for processing batch requests

          priority: Priority for batch processing (optional)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/batches",
            body=maybe_transform(
                {
                    "endpoint": endpoint,
                    "input_file_id": input_file_id,
                    "completion_window": completion_window,
                    "model_id": model_id,
                    "priority": priority,
                },
                batch_create_params.BatchCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BatchCreateResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BatchJob:
        """
        Get details of a batch job by ID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/batches/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BatchJob,
        )

    def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BatchListResponse:
        """List all batch jobs for the authenticated user"""
        return self._get(
            "/batches",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BatchListResponse,
        )

    def cancel(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BatchJob:
        """
        Cancel a batch job by ID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/batches/{id}/cancel",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BatchJob,
        )


class AsyncBatchesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncBatchesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/togethercomputer/together-py#accessing-raw-response-data-eg-headers
        """
        return AsyncBatchesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncBatchesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/togethercomputer/together-py#with_streaming_response
        """
        return AsyncBatchesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        endpoint: str,
        input_file_id: str,
        completion_window: str | Omit = omit,
        model_id: str | Omit = omit,
        priority: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BatchCreateResponse:
        """
        Create a new batch job with the given input file and endpoint

        Args:
          endpoint: The endpoint to use for batch processing

          input_file_id: ID of the uploaded input file containing batch requests

          completion_window: Time window for batch completion (optional)

          model_id: Model to use for processing batch requests

          priority: Priority for batch processing (optional)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/batches",
            body=await async_maybe_transform(
                {
                    "endpoint": endpoint,
                    "input_file_id": input_file_id,
                    "completion_window": completion_window,
                    "model_id": model_id,
                    "priority": priority,
                },
                batch_create_params.BatchCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BatchCreateResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BatchJob:
        """
        Get details of a batch job by ID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/batches/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BatchJob,
        )

    async def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BatchListResponse:
        """List all batch jobs for the authenticated user"""
        return await self._get(
            "/batches",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BatchListResponse,
        )

    async def cancel(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BatchJob:
        """
        Cancel a batch job by ID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/batches/{id}/cancel",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BatchJob,
        )


class BatchesResourceWithRawResponse:
    def __init__(self, batches: BatchesResource) -> None:
        self._batches = batches

        self.create = to_raw_response_wrapper(
            batches.create,
        )
        self.retrieve = to_raw_response_wrapper(
            batches.retrieve,
        )
        self.list = to_raw_response_wrapper(
            batches.list,
        )
        self.cancel = to_raw_response_wrapper(
            batches.cancel,
        )


class AsyncBatchesResourceWithRawResponse:
    def __init__(self, batches: AsyncBatchesResource) -> None:
        self._batches = batches

        self.create = async_to_raw_response_wrapper(
            batches.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            batches.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            batches.list,
        )
        self.cancel = async_to_raw_response_wrapper(
            batches.cancel,
        )


class BatchesResourceWithStreamingResponse:
    def __init__(self, batches: BatchesResource) -> None:
        self._batches = batches

        self.create = to_streamed_response_wrapper(
            batches.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            batches.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            batches.list,
        )
        self.cancel = to_streamed_response_wrapper(
            batches.cancel,
        )


class AsyncBatchesResourceWithStreamingResponse:
    def __init__(self, batches: AsyncBatchesResource) -> None:
        self._batches = batches

        self.create = async_to_streamed_response_wrapper(
            batches.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            batches.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            batches.list,
        )
        self.cancel = async_to_streamed_response_wrapper(
            batches.cancel,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict
from typing_extensions import TypedDict

__all__ = ["ToolsParam", "Function"]


class Function(TypedDict, total=False):
    description: str

    name: str

    parameters: Dict[str, object]
    """A map of parameter names to their values."""


class ToolsParam(TypedDict, total=False):
    function: Function

    type: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Iterable
from typing_extensions import Literal, overload

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import required_args, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._streaming import Stream, AsyncStream
from ...types.chat import completion_create_params
from ..._base_client import make_request_options
from ...types.tools_param import ToolsParam
from ...types.chat.chat_completion import ChatCompletion
from ...types.chat.chat_completion_chunk import ChatCompletionChunk

__all__ = ["CompletionsResource", "AsyncCompletionsResource"]


class CompletionsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> CompletionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/togethercomputer/together-py#accessing-raw-response-data-eg-headers
        """
        return CompletionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CompletionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/togethercomputer/together-py#with_streaming_response
        """
        return CompletionsResourceWithStreamingResponse(self)

    @overload
    def create(
        self,
        *,
        messages: Iterable[completion_create_params.Message],
        model: Union[
            Literal[
                "Qwen/Qwen2.5-72B-Instruct-Turbo",
                "Qwen/Qwen2.5-7B-Instruct-Turbo",
                "meta-llama/Meta-Llama-3.1-405B-Instruct-Turbo",
                "meta-llama/Meta-Llama-3.1-70B-Instruct-Turbo",
                "meta-llama/Meta-Llama-3.1-8B-Instruct-Turbo",
            ],
            str,
        ],
        context_length_exceeded_behavior: Literal["truncate", "error"] | Omit = omit,
        echo: bool | Omit = omit,
        frequency_penalty: float | Omit = omit,
        function_call: completion_create_params.FunctionCall | Omit = omit,
        logit_bias: Dict[str, float] | Omit = omit,
        logprobs: int | Omit = omit,
        max_tokens: int | Omit = omit,
        min_p: float | Omit = omit,
        n: int | Omit = omit,
        presence_penalty: float | Omit = omit,
        reasoning_effort: Literal["low", "medium", "high"] | Omit = omit,
        repetition_penalty: float | Omit = omit,
        response_format: completion_create_params.ResponseFormat | Omit = omit,
        safety_model: str | Omit = omit,
        seed: int | Omit = omit,
        stop: SequenceNotStr[str] | Omit = omit,
        stream: Literal[False] | Omit = omit,
        temperature: float | Omit = omit,
        tool_choice: completion_create_params.ToolChoice | Omit = omit,
        tools: Iterable[ToolsParam] | Omit = omit,
        top_k: int | Omit = omit,
        top_p: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ChatCompletion:
        """
        Query a chat model.

        Args:
          messages: A list of messages comprising the conversation so far.

          model: The name of the model to query.

              [See all of Together AI's chat models](https://docs.together.ai/docs/serverless-models#chat-models)

          context_length_exceeded_behavior: Defined the behavior of the API when max_tokens exceed the maximum context
              length of the model. When set to 'error', API will return 400 with appropriate
              error message. When set to 'truncate', override the max_tokens with maximum
              context length of the model.

          echo: If true, the response will contain the prompt. Can be used with `logprobs` to
              return prompt logprobs.

          frequency_penalty: A number between -2.0 and 2.0 where a positive value decreases the likelihood of
              repeating tokens that have already been mentioned.

          logit_bias: Adjusts the likelihood of specific tokens appearing in the generated output.

          logprobs: An integer between 0 and 20 of the top k tokens to return log probabilities for
              at each generation step, instead of just the sampled token. Log probabilities
              help assess model confidence in token predictions.

          max_tokens: The maximum number of tokens to generate.

          min_p: A number between 0 and 1 that can be used as an alternative to top_p and top-k.

          n: The number of completions to generate for each prompt.

          presence_penalty: A number between -2.0 and 2.0 where a positive value increases the likelihood of
              a model talking about new topics.

          reasoning_effort: Controls the level of reasoning effort the model should apply when generating
              responses. Higher values may result in more thoughtful and detailed responses
              but may take longer to generate.

          repetition_penalty: A number that controls the diversity of generated text by reducing the
              likelihood of repeated sequences. Higher values decrease repetition.

          response_format: An object specifying the format that the model must output.

              Setting to `{ "type": "json_schema", "json_schema": {...} }` enables Structured
              Outputs which ensures the model will match your supplied JSON schema. Learn more
              in the [Structured Outputs guide](https://docs.together.ai/docs/json-mode).

              Setting to `{ "type": "json_object" }` enables the older JSON mode, which
              ensures the message the model generates is valid JSON. Using `json_schema` is
              preferred for models that support it.

          safety_model: The name of the moderation model used to validate tokens. Choose from the
              available moderation models found
              [here](https://docs.together.ai/docs/inference-models#moderation-models).

          seed: Seed value for reproducibility.

          stop: A list of string sequences that will truncate (stop) inference text output. For
              example, "</s>" will stop generation as soon as the model generates the given
              token.

          stream: If true, stream tokens as Server-Sent Events as the model generates them instead
              of waiting for the full model response. The stream terminates with
              `data: [DONE]`. If false, return a single JSON object containing the results.

          temperature: A decimal number from 0-1 that determines the degree of randomness in the
              response. A temperature less than 1 favors more correctness and is appropriate
              for question answering or summarization. A value closer to 1 introduces more
              randomness in the output.

          tool_choice: Controls which (if any) function is called by the model. By default uses `auto`,
              which lets the model pick between generating a message or calling a function.

          tools: A list of tools the model may call. Currently, only functions are supported as a
              tool. Use this to provide a list of functions the model may generate JSON inputs
              for.

          top_k: An integer that's used to limit the number of choices for the next predicted
              word or token. It specifies the maximum number of tokens to consider at each
              step, based on their probability of occurrence. This technique helps to speed up
              the generation process and can improve the quality of the generated text by
              focusing on the most likely options.

          top_p: A percentage (also called the nucleus parameter) that's used to dynamically
              adjust the number of choices for each predicted token based on the cumulative
              probabilities. It specifies a probability threshold below which all less likely
              tokens are filtered out. This technique helps maintain diversity and generate
              more fluent and natural-sounding text.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def create(
        self,
        *,
        messages: Iterable[completion_create_params.Message],
        model: Union[
            Literal[
                "Qwen/Qwen2.5-72B-Instruct-Turbo",
                "Qwen/Qwen2.5-7B-Instruct-Turbo",
                "meta-llama/Meta-Llama-3.1-405B-Instruct-Turbo",
                "meta-llama/Meta-Llama-3.1-70B-Instruct-Turbo",
                "meta-llama/Meta-Llama-3.1-8B-Instruct-Turbo",
            ],
            str,
        ],
        stream: Literal[True],
        context_length_exceeded_behavior: Literal["truncate", "error"] | Omit = omit,
        echo: bool | Omit = omit,
        frequency_penalty: float | Omit = omit,
        function_call: completion_create_params.FunctionCall | Omit = omit,
        logit_bias: Dict[str, float] | Omit = omit,
        logprobs: int | Omit = omit,
        max_tokens: int | Omit = omit,
        min_p: float | Omit = omit,
        n: int | Omit = omit,
        presence_penalty: float | Omit = omit,
        reasoning_effort: Literal["low", "medium", "high"] | Omit = omit,
        repetition_penalty: float | Omit = omit,
        response_format: completion_create_params.ResponseFormat | Omit = omit,
        safety_model: str | Omit = omit,
        seed: int | Omit = omit,
        stop: SequenceNotStr[str] | Omit = omit,
        temperature: float | Omit = omit,
        tool_choice: completion_create_params.ToolChoice | Omit = omit,
        tools: Iterable[ToolsParam] | Omit = omit,
        top_k: int | Omit = omit,
        top_p: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Stream[ChatCompletionChunk]:
        """
        Query a chat model.

        Args:
          messages: A list of messages comprising the conversation so far.

          model: The name of the model to query.

              [See all of Together AI's chat models](https://docs.together.ai/docs/serverless-models#chat-models)

          stream: If true, stream tokens as Server-Sent Events as the model generates them instead
              of waiting for the full model response. The stream terminates with
              `data: [DONE]`. If false, return a single JSON object containing the results.

          context_length_exceeded_behavior: Defined the behavior of the API when max_tokens exceed the maximum context
              length of the model. When set to 'error', API will return 400 with appropriate
              error message. When set to 'truncate', override the max_tokens with maximum
              context length of the model.

          echo: If true, the response will contain the prompt. Can be used with `logprobs` to
              return prompt logprobs.

          frequency_penalty: A number between -2.0 and 2.0 where a positive value decreases the likelihood of
              repeating tokens that have already been mentioned.

          logit_bias: Adjusts the likelihood of specific tokens appearing in the generated output.

          logprobs: An integer between 0 and 20 of the top k tokens to return log probabilities for
              at each generation step, instead of just the sampled token. Log probabilities
              help assess model confidence in token predictions.

          max_tokens: The maximum number of tokens to generate.

          min_p: A number between 0 and 1 that can be used as an alternative to top_p and top-k.

          n: The number of completions to generate for each prompt.

          presence_penalty: A number between -2.0 and 2.0 where a positive value increases the likelihood of
              a model talking about new topics.

          reasoning_effort: Controls the level of reasoning effort the model should apply when generating
              responses. Higher values may result in more thoughtful and detailed responses
              but may take longer to generate.

          repetition_penalty: A number that controls the diversity of generated text by reducing the
              likelihood of repeated sequences. Higher values decrease repetition.

          response_format: An object specifying the format that the model must output.

              Setting to `{ "type": "json_schema", "json_schema": {...} }` enables Structured
              Outputs which ensures the model will match your supplied JSON schema. Learn more
              in the [Structured Outputs guide](https://docs.together.ai/docs/json-mode).

              Setting to `{ "type": "json_object" }` enables the older JSON mode, which
              ensures the message the model generates is valid JSON. Using `json_schema` is
              preferred for models that support it.

          safety_model: The name of the moderation model used to validate tokens. Choose from the
              available moderation models found
              [here](https://docs.together.ai/docs/inference-models#moderation-models).

          seed: Seed value for reproducibility.

          stop: A list of string sequences that will truncate (stop) inference text output. For
              example, "</s>" will stop generation as soon as the model generates the given
              token.

          temperature: A decimal number from 0-1 that determines the degree of randomness in the
              response. A temperature less than 1 favors more correctness and is appropriate
              for question answering or summarization. A value closer to 1 introduces more
              randomness in the output.

          tool_choice: Controls which (if any) function is called by the model. By default uses `auto`,
              which lets the model pick between generating a message or calling a function.

          tools: A list of tools the model may call. Currently, only functions are supported as a
              tool. Use this to provide a list of functions the model may generate JSON inputs
              for.

          top_k: An integer that's used to limit the number of choices for the next predicted
              word or token. It specifies the maximum number of tokens to consider at each
              step, based on their probability of occurrence. This technique helps to speed up
              the generation process and can improve the quality of the generated text by
              focusing on the most likely options.

          top_p: A percentage (also called the nucleus parameter) that's used to dynamically
              adjust the number of choices for each predicted token based on the cumulative
              probabilities. It specifies a probability threshold below which all less likely
              tokens are filtered out. This technique helps maintain diversity and generate
              more fluent and natural-sounding text.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def create(
        self,
        *,
        messages: Iterable[completion_create_params.Message],
        model: Union[
            Literal[
                "Qwen/Qwen2.5-72B-Instruct-Turbo",
                "Qwen/Qwen2.5-7B-Instruct-Turbo",
                "meta-llama/Meta-Llama-3.1-405B-Instruct-Turbo",
                "meta-llama/Meta-Llama-3.1-70B-Instruct-Turbo",
                "meta-llama/Meta-Llama-3.1-8B-Instruct-Turbo",
            ],
            str,
        ],
        stream: bool,
        context_length_exceeded_behavior: Literal["truncate", "error"] | Omit = omit,
        echo: bool | Omit = omit,
        frequency_penalty: float | Omit = omit,
        function_call: completion_create_params.FunctionCall | Omit = omit,
        logit_bias: Dict[str, float] | Omit = omit,
        logprobs: int | Omit = omit,
        max_tokens: int | Omit = omit,
        min_p: float | Omit = omit,
        n: int | Omit = omit,
        presence_penalty: float | Omit = omit,
        reasoning_effort: Literal["low", "medium", "high"] | Omit = omit,
        repetition_penalty: float | Omit = omit,
        response_format: completion_create_params.ResponseFormat | Omit = omit,
        safety_model: str | Omit = omit,
        seed: int | Omit = omit,
        stop: SequenceNotStr[str] | Omit = omit,
        temperature: float | Omit = omit,
        tool_choice: completion_create_params.ToolChoice | Omit = omit,
        tools: Iterable[ToolsParam] | Omit = omit,
        top_k: int | Omit = omit,
        top_p: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ChatCompletion | Stream[ChatCompletionChunk]:
        """
        Query a chat model.

        Args:
          messages: A list of messages comprising the conversation so far.

          model: The name of the model to query.

              [See all of Together AI's chat models](https://docs.together.ai/docs/serverless-models#chat-models)

          stream: If true, stream tokens as Server-Sent Events as the model generates them instead
              of waiting for the full model response. The stream terminates with
              `data: [DONE]`. If false, return a single JSON object containing the results.

          context_length_exceeded_behavior: Defined the behavior of the API when max_tokens exceed the maximum context
              length of the model. When set to 'error', API will return 400 with appropriate
              error message. When set to 'truncate', override the max_tokens with maximum
              context length of the model.

          echo: If true, the response will contain the prompt. Can be used with `logprobs` to
              return prompt logprobs.

          frequency_penalty: A number between -2.0 and 2.0 where a positive value decreases the likelihood of
              repeating tokens that have already been mentioned.

          logit_bias: Adjusts the likelihood of specific tokens appearing in the generated output.

          logprobs: An integer between 0 and 20 of the top k tokens to return log probabilities for
              at each generation step, instead of just the sampled token. Log probabilities
              help assess model confidence in token predictions.

          max_tokens: The maximum number of tokens to generate.

          min_p: A number between 0 and 1 that can be used as an alternative to top_p and top-k.

          n: The number of completions to generate for each prompt.

          presence_penalty: A number between -2.0 and 2.0 where a positive value increases the likelihood of
              a model talking about new topics.

          reasoning_effort: Controls the level of reasoning effort the model should apply when generating
              responses. Higher values may result in more thoughtful and detailed responses
              but may take longer to generate.

          repetition_penalty: A number that controls the diversity of generated text by reducing the
              likelihood of repeated sequences. Higher values decrease repetition.

          response_format: An object specifying the format that the model must output.

              Setting to `{ "type": "json_schema", "json_schema": {...} }` enables Structured
              Outputs which ensures the model will match your supplied JSON schema. Learn more
              in the [Structured Outputs guide](https://docs.together.ai/docs/json-mode).

              Setting to `{ "type": "json_object" }` enables the older JSON mode, which
              ensures the message the model generates is valid JSON. Using `json_schema` is
              preferred for models that support it.

          safety_model: The name of the moderation model used to validate tokens. Choose from the
              available moderation models found
              [here](https://docs.together.ai/docs/inference-models#moderation-models).

          seed: Seed value for reproducibility.

          stop: A list of string sequences that will truncate (stop) inference text output. For
              example, "</s>" will stop generation as soon as the model generates the given
              token.

          temperature: A decimal number from 0-1 that determines the degree of randomness in the
              response. A temperature less than 1 favors more correctness and is appropriate
              for question answering or summarization. A value closer to 1 introduces more
              randomness in the output.

          tool_choice: Controls which (if any) function is called by the model. By default uses `auto`,
              which lets the model pick between generating a message or calling a function.

          tools: A list of tools the model may call. Currently, only functions are supported as a
              tool. Use this to provide a list of functions the model may generate JSON inputs
              for.

          top_k: An integer that's used to limit the number of choices for the next predicted
              word or token. It specifies the maximum number of tokens to consider at each
              step, based on their probability of occurrence. This technique helps to speed up
              the generation process and can improve the quality of the generated text by
              focusing on the most likely options.

          top_p: A percentage (also called the nucleus parameter) that's used to dynamically
              adjust the number of choices for each predicted token based on the cumulative
              probabilities. It specifies a probability threshold below which all less likely
              tokens are filtered out. This technique helps maintain diversity and generate
              more fluent and natural-sounding text.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["messages", "model"], ["messages", "model", "stream"])
    def create(
        self,
        *,
        messages: Iterable[completion_create_params.Message],
        model: Union[
            Literal[
                "Qwen/Qwen2.5-72B-Instruct-Turbo",
                "Qwen/Qwen2.5-7B-Instruct-Turbo",
                "meta-llama/Meta-Llama-3.1-405B-Instruct-Turbo",
                "meta-llama/Meta-Llama-3.1-70B-Instruct-Turbo",
                "meta-llama/Meta-Llama-3.1-8B-Instruct-Turbo",
            ],
            str,
        ],
        context_length_exceeded_behavior: Literal["truncate", "error"] | Omit = omit,
        echo: bool | Omit = omit,
        frequency_penalty: float | Omit = omit,
        function_call: completion_create_params.FunctionCall | Omit = omit,
        logit_bias: Dict[str, float] | Omit = omit,
        logprobs: int | Omit = omit,
        max_tokens: int | Omit = omit,
        min_p: float | Omit = omit,
        n: int | Omit = omit,
        presence_penalty: float | Omit = omit,
        reasoning_effort: Literal["low", "medium", "high"] | Omit = omit,
        repetition_penalty: float | Omit = omit,
        response_format: completion_create_params.ResponseFormat | Omit = omit,
        safety_model: str | Omit = omit,
        seed: int | Omit = omit,
        stop: SequenceNotStr[str] | Omit = omit,
        stream: Literal[False] | Literal[True] | Omit = omit,
        temperature: float | Omit = omit,
        tool_choice: completion_create_params.ToolChoice | Omit = omit,
        tools: Iterable[ToolsParam] | Omit = omit,
        top_k: int | Omit = omit,
        top_p: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ChatCompletion | Stream[ChatCompletionChunk]:
        return self._post(
            "/chat/completions",
            body=maybe_transform(
                {
                    "messages": messages,
                    "model": model,
                    "context_length_exceeded_behavior": context_length_exceeded_behavior,
                    "echo": echo,
                    "frequency_penalty": frequency_penalty,
                    "function_call": function_call,
                    "logit_bias": logit_bias,
                    "logprobs": logprobs,
                    "max_tokens": max_tokens,
                    "min_p": min_p,
                    "n": n,
                    "presence_penalty": presence_penalty,
                    "reasoning_effort": reasoning_effort,
                    "repetition_penalty": repetition_penalty,
                    "response_format": response_format,
                    "safety_model": safety_model,
                    "seed": seed,
                    "stop": stop,
                    "stream": stream,
                    "temperature": temperature,
                    "tool_choice": tool_choice,
                    "tools": tools,
                    "top_k": top_k,
                    "top_p": top_p,
                },
                completion_create_params.CompletionCreateParamsStreaming
                if stream
                else completion_create_params.CompletionCreateParamsNonStreaming,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ChatCompletion,
            stream=stream or False,
            stream_cls=Stream[ChatCompletionChunk],
        )


class AsyncCompletionsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncCompletionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/togethercomputer/together-py#accessing-raw-response-data-eg-headers
        """
        return AsyncCompletionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCompletionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/togethercomputer/together-py#with_streaming_response
        """
        return AsyncCompletionsResourceWithStreamingResponse(self)

    @overload
    async def create(
        self,
        *,
        messages: Iterable[completion_create_params.Message],
        model: Union[
            Literal[
                "Qwen/Qwen2.5-72B-Instruct-Turbo",
                "Qwen/Qwen2.5-7B-Instruct-Turbo",
                "meta-llama/Meta-Llama-3.1-405B-Instruct-Turbo",
                "meta-llama/Meta-Llama-3.1-70B-Instruct-Turbo",
                "meta-llama/Meta-Llama-3.1-8B-Instruct-Turbo",
            ],
            str,
        ],
        context_length_exceeded_behavior: Literal["truncate", "error"] | Omit = omit,
        echo: bool | Omit = omit,
        frequency_penalty: float | Omit = omit,
        function_call: completion_create_params.FunctionCall | Omit = omit,
        logit_bias: Dict[str, float] | Omit = omit,
        logprobs: int | Omit = omit,
        max_tokens: int | Omit = omit,
        min_p: float | Omit = omit,
        n: int | Omit = omit,
        presence_penalty: float | Omit = omit,
        reasoning_effort: Literal["low", "medium", "high"] | Omit = omit,
        repetition_penalty: float | Omit = omit,
        response_format: completion_create_params.ResponseFormat | Omit = omit,
        safety_model: str | Omit = omit,
        seed: int | Omit = omit,
        stop: SequenceNotStr[str] | Omit = omit,
        stream: Literal[False] | Omit = omit,
        temperature: float | Omit = omit,
        tool_choice: completion_create_params.ToolChoice | Omit = omit,
        tools: Iterable[ToolsParam] | Omit = omit,
        top_k: int | Omit = omit,
        top_p: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ChatCompletion:
        """
        Query a chat model.

        Args:
          messages: A list of messages comprising the conversation so far.

          model: The name of the model to query.

              [See all of Together AI's chat models](https://docs.together.ai/docs/serverless-models#chat-models)

          context_length_exceeded_behavior: Defined the behavior of the API when max_tokens exceed the maximum context
              length of the model. When set to 'error', API will return 400 with appropriate
              error message. When set to 'truncate', override the max_tokens with maximum
              context length of the model.

          echo: If true, the response will contain the prompt. Can be used with `logprobs` to
              return prompt logprobs.

          frequency_penalty: A number between -2.0 and 2.0 where a positive value decreases the likelihood of
              repeating tokens that have already been mentioned.

          logit_bias: Adjusts the likelihood of specific tokens appearing in the generated output.

          logprobs: An integer between 0 and 20 of the top k tokens to return log probabilities for
              at each generation step, instead of just the sampled token. Log probabilities
              help assess model confidence in token predictions.

          max_tokens: The maximum number of tokens to generate.

          min_p: A number between 0 and 1 that can be used as an alternative to top_p and top-k.

          n: The number of completions to generate for each prompt.

          presence_penalty: A number between -2.0 and 2.0 where a positive value increases the likelihood of
              a model talking about new topics.

          reasoning_effort: Controls the level of reasoning effort the model should apply when generating
              responses. Higher values may result in more thoughtful and detailed responses
              but may take longer to generate.

          repetition_penalty: A number that controls the diversity of generated text by reducing the
              likelihood of repeated sequences. Higher values decrease repetition.

          response_format: An object specifying the format that the model must output.

              Setting to `{ "type": "json_schema", "json_schema": {...} }` enables Structured
              Outputs which ensures the model will match your supplied JSON schema. Learn more
              in the [Structured Outputs guide](https://docs.together.ai/docs/json-mode).

              Setting to `{ "type": "json_object" }` enables the older JSON mode, which
              ensures the message the model generates is valid JSON. Using `json_schema` is
              preferred for models that support it.

          safety_model: The name of the moderation model used to validate tokens. Choose from the
              available moderation models found
              [here](https://docs.together.ai/docs/inference-models#moderation-models).

          seed: Seed value for reproducibility.

          stop: A list of string sequences that will truncate (stop) inference text output. For
              example, "</s>" will stop generation as soon as the model generates the given
              token.

          stream: If true, stream tokens as Server-Sent Events as the model generates them instead
              of waiting for the full model response. The stream terminates with
              `data: [DONE]`. If false, return a single JSON object containing the results.

          temperature: A decimal number from 0-1 that determines the degree of randomness in the
              response. A temperature less than 1 favors more correctness and is appropriate
              for question answering or summarization. A value closer to 1 introduces more
              randomness in the output.

          tool_choice: Controls which (if any) function is called by the model. By default uses `auto`,
              which lets the model pick between generating a message or calling a function.

          tools: A list of tools the model may call. Currently, only functions are supported as a
              tool. Use this to provide a list of functions the model may generate JSON inputs
              for.

          top_k: An integer that's used to limit the number of choices for the next predicted
              word or token. It specifies the maximum number of tokens to consider at each
              step, based on their probability of occurrence. This technique helps to speed up
              the generation process and can improve the quality of the generated text by
              focusing on the most likely options.

          top_p: A percentage (also called the nucleus parameter) that's used to dynamically
              adjust the number of choices for each predicted token based on the cumulative
              probabilities. It specifies a probability threshold below which all less likely
              tokens are filtered out. This technique helps maintain diversity and generate
              more fluent and natural-sounding text.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def create(
        self,
        *,
        messages: Iterable[completion_create_params.Message],
        model: Union[
            Literal[
                "Qwen/Qwen2.5-72B-Instruct-Turbo",
                "Qwen/Qwen2.5-7B-Instruct-Turbo",
                "meta-llama/Meta-Llama-3.1-405B-Instruct-Turbo",
                "meta-llama/Meta-Llama-3.1-70B-Instruct-Turbo",
                "meta-llama/Meta-Llama-3.1-8B-Instruct-Turbo",
            ],
            str,
        ],
        stream: Literal[True],
        context_length_exceeded_behavior: Literal["truncate", "error"] | Omit = omit,
        echo: bool | Omit = omit,
        frequency_penalty: float | Omit = omit,
        function_call: completion_create_params.FunctionCall | Omit = omit,
        logit_bias: Dict[str, float] | Omit = omit,
        logprobs: int | Omit = omit,
        max_tokens: int | Omit = omit,
        min_p: float | Omit = omit,
        n: int | Omit = omit,
        presence_penalty: float | Omit = omit,
        reasoning_effort: Literal["low", "medium", "high"] | Omit = omit,
        repetition_penalty: float | Omit = omit,
        response_format: completion_create_params.ResponseFormat | Omit = omit,
        safety_model: str | Omit = omit,
        seed: int | Omit = omit,
        stop: SequenceNotStr[str] | Omit = omit,
        temperature: float | Omit = omit,
        tool_choice: completion_create_params.ToolChoice | Omit = omit,
        tools: Iterable[ToolsParam] | Omit = omit,
        top_k: int | Omit = omit,
        top_p: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncStream[ChatCompletionChunk]:
        """
        Query a chat model.

        Args:
          messages: A list of messages comprising the conversation so far.

          model: The name of the model to query.

              [See all of Together AI's chat models](https://docs.together.ai/docs/serverless-models#chat-models)

          stream: If true, stream tokens as Server-Sent Events as the model generates them instead
              of waiting for the full model response. The stream terminates with
              `data: [DONE]`. If false, return a single JSON object containing the results.

          context_length_exceeded_behavior: Defined the behavior of the API when max_tokens exceed the maximum context
              length of the model. When set to 'error', API will return 400 with appropriate
              error message. When set to 'truncate', override the max_tokens with maximum
              context length of the model.

          echo: If true, the response will contain the prompt. Can be used with `logprobs` to
              return prompt logprobs.

          frequency_penalty: A number between -2.0 and 2.0 where a positive value decreases the likelihood of
              repeating tokens that have already been mentioned.

          logit_bias: Adjusts the likelihood of specific tokens appearing in the generated output.

          logprobs: An integer between 0 and 20 of the top k tokens to return log probabilities for
              at each generation step, instead of just the sampled token. Log probabilities
              help assess model confidence in token predictions.

          max_tokens: The maximum number of tokens to generate.

          min_p: A number between 0 and 1 that can be used as an alternative to top_p and top-k.

          n: The number of completions to generate for each prompt.

          presence_penalty: A number between -2.0 and 2.0 where a positive value increases the likelihood of
              a model talking about new topics.

          reasoning_effort: Controls the level of reasoning effort the model should apply when generating
              responses. Higher values may result in more thoughtful and detailed responses
              but may take longer to generate.

          repetition_penalty: A number that controls the diversity of generated text by reducing the
              likelihood of repeated sequences. Higher values decrease repetition.

          response_format: An object specifying the format that the model must output.

              Setting to `{ "type": "json_schema", "json_schema": {...} }` enables Structured
              Outputs which ensures the model will match your supplied JSON schema. Learn more
              in the [Structured Outputs guide](https://docs.together.ai/docs/json-mode).

              Setting to `{ "type": "json_object" }` enables the older JSON mode, which
              ensures the message the model generates is valid JSON. Using `json_schema` is
              preferred for models that support it.

          safety_model: The name of the moderation model used to validate tokens. Choose from the
              available moderation models found
              [here](https://docs.together.ai/docs/inference-models#moderation-models).

          seed: Seed value for reproducibility.

          stop: A list of string sequences that will truncate (stop) inference text output. For
              example, "</s>" will stop generation as soon as the model generates the given
              token.

          temperature: A decimal number from 0-1 that determines the degree of randomness in the
              response. A temperature less than 1 favors more correctness and is appropriate
              for question answering or summarization. A value closer to 1 introduces more
              randomness in the output.

          tool_choice: Controls which (if any) function is called by the model. By default uses `auto`,
              which lets the model pick between generating a message or calling a function.

          tools: A list of tools the model may call. Currently, only functions are supported as a
              tool. Use this to provide a list of functions the model may generate JSON inputs
              for.

          top_k: An integer that's used to limit the number of choices for the next predicted
              word or token. It specifies the maximum number of tokens to consider at each
              step, based on their probability of occurrence. This technique helps to speed up
              the generation process and can improve the quality of the generated text by
              focusing on the most likely options.

          top_p: A percentage (also called the nucleus parameter) that's used to dynamically
              adjust the number of choices for each predicted token based on the cumulative
              probabilities. It specifies a probability threshold below which all less likely
              tokens are filtered out. This technique helps maintain diversity and generate
              more fluent and natural-sounding text.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def create(
        self,
        *,
        messages: Iterable[completion_create_params.Message],
        model: Union[
            Literal[
                "Qwen/Qwen2.5-72B-Instruct-Turbo",
                "Qwen/Qwen2.5-7B-Instruct-Turbo",
                "meta-llama/Meta-Llama-3.1-405B-Instruct-Turbo",
                "meta-llama/Meta-Llama-3.1-70B-Instruct-Turbo",
                "meta-llama/Meta-Llama-3.1-8B-Instruct-Turbo",
            ],
            str,
        ],
        stream: bool,
        context_length_exceeded_behavior: Literal["truncate", "error"] | Omit = omit,
        echo: bool | Omit = omit,
        frequency_penalty: float | Omit = omit,
        function_call: completion_create_params.FunctionCall | Omit = omit,
        logit_bias: Dict[str, float] | Omit = omit,
        logprobs: int | Omit = omit,
        max_tokens: int | Omit = omit,
        min_p: float | Omit = omit,
        n: int | Omit = omit,
        presence_penalty: float | Omit = omit,
        reasoning_effort: Literal["low", "medium", "high"] | Omit = omit,
        repetition_penalty: float | Omit = omit,
        response_format: completion_create_params.ResponseFormat | Omit = omit,
        safety_model: str | Omit = omit,
        seed: int | Omit = omit,
        stop: SequenceNotStr[str] | Omit = omit,
        temperature: float | Omit = omit,
        tool_choice: completion_create_params.ToolChoice | Omit = omit,
        tools: Iterable[ToolsParam] | Omit = omit,
        top_k: int | Omit = omit,
        top_p: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ChatCompletion | AsyncStream[ChatCompletionChunk]:
        """
        Query a chat model.

        Args:
          messages: A list of messages comprising the conversation so far.

          model: The name of the model to query.

              [See all of Together AI's chat models](https://docs.together.ai/docs/serverless-models#chat-models)

          stream: If true, stream tokens as Server-Sent Events as the model generates them instead
              of waiting for the full model response. The stream terminates with
              `data: [DONE]`. If false, return a single JSON object containing the results.

          context_length_exceeded_behavior: Defined the behavior of the API when max_tokens exceed the maximum context
              length of the model. When set to 'error', API will return 400 with appropriate
              error message. When set to 'truncate', override the max_tokens with maximum
              context length of the model.

          echo: If true, the response will contain the prompt. Can be used with `logprobs` to
              return prompt logprobs.

          frequency_penalty: A number between -2.0 and 2.0 where a positive value decreases the likelihood of
              repeating tokens that have already been mentioned.

          logit_bias: Adjusts the likelihood of specific tokens appearing in the generated output.

          logprobs: An integer between 0 and 20 of the top k tokens to return log probabilities for
              at each generation step, instead of just the sampled token. Log probabilities
              help assess model confidence in token predictions.

          max_tokens: The maximum number of tokens to generate.

          min_p: A number between 0 and 1 that can be used as an alternative to top_p and top-k.

          n: The number of completions to generate for each prompt.

          presence_penalty: A number between -2.0 and 2.0 where a positive value increases the likelihood of
              a model talking about new topics.

          reasoning_effort: Controls the level of reasoning effort the model should apply when generating
              responses. Higher values may result in more thoughtful and detailed responses
              but may take longer to generate.

          repetition_penalty: A number that controls the diversity of generated text by reducing the
              likelihood of repeated sequences. Higher values decrease repetition.

          response_format: An object specifying the format that the model must output.

              Setting to `{ "type": "json_schema", "json_schema": {...} }` enables Structured
              Outputs which ensures the model will match your supplied JSON schema. Learn more
              in the [Structured Outputs guide](https://docs.together.ai/docs/json-mode).

              Setting to `{ "type": "json_object" }` enables the older JSON mode, which
              ensures the message the model generates is valid JSON. Using `json_schema` is
              preferred for models that support it.

          safety_model: The name of the moderation model used to validate tokens. Choose from the
              available moderation models found
              [here](https://docs.together.ai/docs/inference-models#moderation-models).

          seed: Seed value for reproducibility.

          stop: A list of string sequences that will truncate (stop) inference text output. For
              example, "</s>" will stop generation as soon as the model generates the given
              token.

          temperature: A decimal number from 0-1 that determines the degree of randomness in the
              response. A temperature less than 1 favors more correctness and is appropriate
              for question answering or summarization. A value closer to 1 introduces more
              randomness in the output.

          tool_choice: Controls which (if any) function is called by the model. By default uses `auto`,
              which lets the model pick between generating a message or calling a function.

          tools: A list of tools the model may call. Currently, only functions are supported as a
              tool. Use this to provide a list of functions the model may generate JSON inputs
              for.

          top_k: An integer that's used to limit the number of choices for the next predicted
              word or token. It specifies the maximum number of tokens to consider at each
              step, based on their probability of occurrence. This technique helps to speed up
              the generation process and can improve the quality of the generated text by
              focusing on the most likely options.

          top_p: A percentage (also called the nucleus parameter) that's used to dynamically
              adjust the number of choices for each predicted token based on the cumulative
              probabilities. It specifies a probability threshold below which all less likely
              tokens are filtered out. This technique helps maintain diversity and generate
              more fluent and natural-sounding text.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["messages", "model"], ["messages", "model", "stream"])
    async def create(
        self,
        *,
        messages: Iterable[completion_create_params.Message],
        model: Union[
            Literal[
                "Qwen/Qwen2.5-72B-Instruct-Turbo",
                "Qwen/Qwen2.5-7B-Instruct-Turbo",
                "meta-llama/Meta-Llama-3.1-405B-Instruct-Turbo",
                "meta-llama/Meta-Llama-3.1-70B-Instruct-Turbo",
                "meta-llama/Meta-Llama-3.1-8B-Instruct-Turbo",
            ],
            str,
        ],
        context_length_exceeded_behavior: Literal["truncate", "error"] | Omit = omit,
        echo: bool | Omit = omit,
        frequency_penalty: float | Omit = omit,
        function_call: completion_create_params.FunctionCall | Omit = omit,
        logit_bias: Dict[str, float] | Omit = omit,
        logprobs: int | Omit = omit,
        max_tokens: int | Omit = omit,
        min_p: float | Omit = omit,
        n: int | Omit = omit,
        presence_penalty: float | Omit = omit,
        reasoning_effort: Literal["low", "medium", "high"] | Omit = omit,
        repetition_penalty: float | Omit = omit,
        response_format: completion_create_params.ResponseFormat | Omit = omit,
        safety_model: str | Omit = omit,
        seed: int | Omit = omit,
        stop: SequenceNotStr[str] | Omit = omit,
        stream: Literal[False] | Literal[True] | Omit = omit,
        temperature: float | Omit = omit,
        tool_choice: completion_create_params.ToolChoice | Omit = omit,
        tools: Iterable[ToolsParam] | Omit = omit,
        top_k: int | Omit = omit,
        top_p: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ChatCompletion | AsyncStream[ChatCompletionChunk]:
        return await self._post(
            "/chat/completions",
            body=await async_maybe_transform(
                {
                    "messages": messages,
                    "model": model,
                    "context_length_exceeded_behavior": context_length_exceeded_behavior,
                    "echo": echo,
                    "frequency_penalty": frequency_penalty,
                    "function_call": function_call,
                    "logit_bias": logit_bias,
                    "logprobs": logprobs,
                    "max_tokens": max_tokens,
                    "min_p": min_p,
                    "n": n,
                    "presence_penalty": presence_penalty,
                    "reasoning_effort": reasoning_effort,
                    "repetition_penalty": repetition_penalty,
                    "response_format": response_format,
                    "safety_model": safety_model,
                    "seed": seed,
                    "stop": stop,
                    "stream": stream,
                    "temperature": temperature,
                    "tool_choice": tool_choice,
                    "tools": tools,
                    "top_k": top_k,
                    "top_p": top_p,
                },
                completion_create_params.CompletionCreateParamsStreaming
                if stream
                else completion_create_params.CompletionCreateParamsNonStreaming,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ChatCompletion,
            stream=stream or False,
            stream_cls=AsyncStream[ChatCompletionChunk],
        )


class CompletionsResourceWithRawResponse:
    def __init__(self, completions: CompletionsResource) -> None:
        self._completions = completions

        self.create = to_raw_response_wrapper(
            completions.create,
        )


class AsyncCompletionsResourceWithRawResponse:
    def __init__(self, completions: AsyncCompletionsResource) -> None:
        self._completions = completions

        self.create = async_to_raw_response_wrapper(
            completions.create,
        )


class CompletionsResourceWithStreamingResponse:
    def __init__(self, completions: CompletionsResource) -> None:
        self._completions = completions

        self.create = to_streamed_response_wrapper(
            completions.create,
        )


class AsyncCompletionsResourceWithStreamingResponse:
    def __init__(self, completions: AsyncCompletionsResource) -> None:
        self._completions = completions

        self.create = async_to_streamed_response_wrapper(
            completions.create,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx
from rich import print as rprint

from ..types import fine_tuning_delete_params, fine_tuning_content_params, fine_tuning_estimate_price_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    BinaryAPIResponse,
    AsyncBinaryAPIResponse,
    StreamedBinaryAPIResponse,
    AsyncStreamedBinaryAPIResponse,
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    to_custom_raw_response_wrapper,
    async_to_streamed_response_wrapper,
    to_custom_streamed_response_wrapper,
    async_to_custom_raw_response_wrapper,
    async_to_custom_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..lib.types.fine_tuning import (
    FinetuneResponse as FinetuneResponseLib,
    FinetuneTrainingLimits,
)
from ..types.finetune_response import FinetuneResponse
from ..lib.resources.fine_tuning import (
    get_model_limits,
    async_get_model_limits,
    create_finetune_request,
)
from ..types.fine_tuning_list_response import FineTuningListResponse
from ..types.fine_tuning_cancel_response import FineTuningCancelResponse
from ..types.fine_tuning_delete_response import FineTuningDeleteResponse
from ..types.fine_tuning_list_events_response import FineTuningListEventsResponse
from ..types.fine_tuning_estimate_price_response import FineTuningEstimatePriceResponse
from ..types.fine_tuning_list_checkpoints_response import FineTuningListCheckpointsResponse

__all__ = ["FineTuningResource", "AsyncFineTuningResource"]

_WARNING_MESSAGE_INSUFFICIENT_FUNDS = (
    "The estimated price of the fine-tuning job is {} which is significantly "
    "greater than your current credit limit and balance combined. "
    "It will likely get cancelled due to insufficient funds. "
    "Proceed at your own risk."
)

class FineTuningResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> FineTuningResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/togethercomputer/together-py#accessing-raw-response-data-eg-headers
        """
        return FineTuningResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> FineTuningResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/togethercomputer/together-py#with_streaming_response
        """
        return FineTuningResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        training_file: str,
        model: str | None = None,
        n_epochs: int = 1,
        validation_file: str | None = "",
        n_evals: int | None = 0,
        n_checkpoints: int | None = 1,
        batch_size: int | Literal["max"] = "max",
        learning_rate: float | None = 0.00001,
        lr_scheduler_type: Literal["linear", "cosine"] = "cosine",
        min_lr_ratio: float = 0.0,
        scheduler_num_cycles: float = 0.5,
        warmup_ratio: float = 0.0,
        max_grad_norm: float = 1.0,
        weight_decay: float = 0.0,
        lora: bool = True,
        lora_r: int | None = None,
        lora_dropout: float | None = 0,
        lora_alpha: float | None = None,
        lora_trainable_modules: str | None = "all-linear",
        suffix: str | None = None,
        wandb_api_key: str | None = None,
        wandb_base_url: str | None = None,
        wandb_project_name: str | None = None,
        wandb_name: str | None = None,
        verbose: bool = False,
        model_limits: FinetuneTrainingLimits | None = None,
        train_on_inputs: bool | Literal["auto"] | None = None,
        training_method: str = "sft",
        dpo_beta: float | None = None,
        dpo_normalize_logratios_by_length: bool = False,
        rpo_alpha: float | None = None,
        simpo_gamma: float | None = None,
        from_checkpoint: str | None = None,
        from_hf_model: str | None = None,
        hf_model_revision: str | None = None,
        hf_api_token: str | None = None,
        hf_output_repo_name: str | None = None,
    ) -> FinetuneResponseLib:
        """
        Method to initiate a fine-tuning job

        Args:
            training_file (str): File-ID of a file uploaded to the Together API
            model (str, optional): Name of the base model to run fine-tune job on
            n_epochs (int, optional): Number of epochs for fine-tuning. Defaults to 1.
            validation file (str, optional): File ID of a file uploaded to the Together API for validation.
            n_evals (int, optional): Number of evaluation loops to run. Defaults to 0.
            n_checkpoints (int, optional): Number of checkpoints to save during fine-tuning.
                Defaults to 1.
            batch_size (int or "max"): Batch size for fine-tuning. Defaults to max.
            learning_rate (float, optional): Learning rate multiplier to use for training
                Defaults to 0.00001.
            lr_scheduler_type (Literal["linear", "cosine"]): Learning rate scheduler type. Defaults to "cosine".
            min_lr_ratio (float, optional): Min learning rate ratio of the initial learning rate for
                the learning rate scheduler. Defaults to 0.0.
            scheduler_num_cycles (float, optional): Number or fraction of cycles for the cosine learning rate scheduler. Defaults to 0.5.
            warmup_ratio (float, optional): Warmup ratio for the learning rate scheduler.
            max_grad_norm (float, optional): Max gradient norm. Defaults to 1.0, set to 0 to disable.
            weight_decay (float, optional): Weight decay. Defaults to 0.0.
            lora (bool, optional): Whether to use LoRA adapters. Defaults to True.
            lora_r (int, optional): Rank of LoRA adapters. Defaults to 8.
            lora_dropout (float, optional): Dropout rate for LoRA adapters. Defaults to 0.
            lora_alpha (float, optional): Alpha for LoRA adapters. Defaults to 8.
            lora_trainable_modules (str, optional): Trainable modules for LoRA adapters. Defaults to "all-linear".
            suffix (str, optional): Up to 40 character suffix that will be added to your fine-tuned model name.
                Defaults to None.
            wandb_api_key (str, optional): API key for Weights & Biases integration.
                Defaults to None.
            wandb_base_url (str, optional): Base URL for Weights & Biases integration.
                Defaults to None.
            wandb_project_name (str, optional): Project name for Weights & Biases integration.
                Defaults to None.
            wandb_name (str, optional): Run name for Weights & Biases integration.
                Defaults to None.
            verbose (bool, optional): whether to print the job parameters before submitting a request.
                Defaults to False.
            model_limits (FinetuneTrainingLimits, optional): Limits for the hyperparameters the model in Fine-tuning.
                Defaults to None.
            train_on_inputs (bool or "auto", optional): Whether to mask the user messages in conversational data or prompts in instruction data.
                "auto" will automatically determine whether to mask the inputs based on the data format.
                For datasets with the "text" field (general format), inputs will not be masked.
                For datasets with the "messages" field (conversational format) or "prompt" and "completion" fields
                (Instruction format), inputs will be masked.
                Defaults to None, or "auto" if training_method is "sft" (set in create_finetune_request).
            training_method (str, optional): Training method. Defaults to "sft".
                Supported methods: "sft", "dpo".
            dpo_beta (float, optional): DPO beta parameter. Defaults to None.
            dpo_normalize_logratios_by_length (bool): Whether or not normalize logratios by sample length. Defaults to False,
            rpo_alpha (float, optional): RPO alpha parameter of DPO training to include NLL in the loss. Defaults to None.
            simpo_gamma: (float, optional): SimPO gamma parameter. Defaults to None.
            from_checkpoint (str, optional): The checkpoint identifier to continue training from a previous fine-tuning job.
                The format: {$JOB_ID/$OUTPUT_MODEL_NAME}:{$STEP}.
                The step value is optional, without it the final checkpoint will be used.
            from_hf_model (str, optional): The Hugging Face Hub repo to start training from.
                Should be as close as possible to the base model (specified by the `model` argument) in terms of architecture and size.
            hf_model_revision (str, optional): The revision of the Hugging Face Hub model to continue training from. Defaults to None.
                Example: hf_model_revision=None (defaults to the latest revision in `main`) or
                hf_model_revision="607a30d783dfa663caf39e06633721c8d4cfcd7e" (specific commit).
            hf_api_token (str, optional): API key for the Hugging Face Hub. Defaults to None.
            hf_output_repo_name (str, optional): HF repo to upload the fine-tuned model to. Defaults to None.

        Returns:
            FinetuneResponse: Object containing information about fine-tuning job.
        """

        if model_limits is None:
            model_name = None
            # mypy doesn't understand that model or from_checkpoint is not None
            if model is not None:
                model_name = model
            elif from_checkpoint is not None:
                model_name = from_checkpoint.split(":")[0]
            else:
                # this branch is unreachable, but mypy doesn't know that
                pass
            model_limits = get_model_limits(self._client, str(model_name))

        finetune_request, training_type_cls, training_method_cls = create_finetune_request(
            model_limits=model_limits,
            training_file=training_file,
            model=model,
            n_epochs=n_epochs,
            validation_file=validation_file,
            n_evals=n_evals,
            n_checkpoints=n_checkpoints,
            batch_size=batch_size,
            learning_rate=learning_rate,
            lr_scheduler_type=lr_scheduler_type,
            min_lr_ratio=min_lr_ratio,
            scheduler_num_cycles=scheduler_num_cycles,
            warmup_ratio=warmup_ratio,
            max_grad_norm=max_grad_norm,
            weight_decay=weight_decay,
            lora=lora,
            lora_r=lora_r,
            lora_dropout=lora_dropout,
            lora_alpha=lora_alpha,
            lora_trainable_modules=lora_trainable_modules,
            suffix=suffix,
            wandb_api_key=wandb_api_key,
            wandb_base_url=wandb_base_url,
            wandb_project_name=wandb_project_name,
            wandb_name=wandb_name,
            train_on_inputs=train_on_inputs,
            training_method=training_method,
            dpo_beta=dpo_beta,
            dpo_normalize_logratios_by_length=dpo_normalize_logratios_by_length,
            rpo_alpha=rpo_alpha,
            simpo_gamma=simpo_gamma,
            from_checkpoint=from_checkpoint,
            from_hf_model=from_hf_model,
            hf_model_revision=hf_model_revision,
            hf_api_token=hf_api_token,
            hf_output_repo_name=hf_output_repo_name,
        )


        price_estimation_result = self.estimate_price(
            training_file=training_file,
            from_checkpoint=from_checkpoint or Omit(),
            validation_file=validation_file or Omit(),
            model=model or "",
            n_epochs=finetune_request.n_epochs,
            n_evals=finetune_request.n_evals or 0,
            training_type=training_type_cls,
            training_method=training_method_cls,
        )


        if verbose:
            rprint(
                "Submitting a fine-tuning job with the following parameters:",
                finetune_request,
            )
            if not price_estimation_result.allowed_to_proceed:
                rprint(
                    "[red]"
                    + _WARNING_MESSAGE_INSUFFICIENT_FUNDS.format(
                        price_estimation_result.estimated_total_price # pyright: ignore[reportPossiblyUnboundVariable]
                    )
                    + "[/red]",
                )
        parameter_payload = finetune_request.model_dump(exclude_none=True)

        return self._client.post(
            "/fine-tunes",
            body=parameter_payload,
            cast_to=FinetuneResponseLib,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FinetuneResponse:
        """
        List the metadata for a single fine-tuning job.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/fine-tunes/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FinetuneResponse,
        )

    def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FineTuningListResponse:
        """List the metadata for all fine-tuning jobs.

        Returns a list of
        FinetuneResponseTruncated objects.
        """
        return self._get(
            "/fine-tunes",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FineTuningListResponse,
        )

    def delete(
        self,
        id: str,
        *,
        force: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FineTuningDeleteResponse:
        """
        Delete a fine-tuning job.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/fine-tunes/{id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"force": force}, fine_tuning_delete_params.FineTuningDeleteParams),
            ),
            cast_to=FineTuningDeleteResponse,
        )

    def cancel(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FineTuningCancelResponse:
        """Cancel a currently running fine-tuning job.

        Returns a FinetuneResponseTruncated
        object.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/fine-tunes/{id}/cancel",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FineTuningCancelResponse,
        )

    def content(
        self,
        *,
        ft_id: str,
        checkpoint: Literal["merged", "adapter", "model_output_path"] | Omit = omit,
        checkpoint_step: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BinaryAPIResponse:
        """
        Receive a compressed fine-tuned model or checkpoint.

        Args:
          ft_id: Fine-tune ID to download. A string that starts with `ft-`.

          checkpoint: Specifies checkpoint type to download - `merged` vs `adapter`. This field is
              required if the checkpoint_step is not set.

          checkpoint_step: Specifies step number for checkpoint to download. Ignores `checkpoint` value if
              set.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "application/octet-stream", **(extra_headers or {})}
        return self._get(
            "/finetune/download",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "ft_id": ft_id,
                        "checkpoint": checkpoint,
                        "checkpoint_step": checkpoint_step,
                    },
                    fine_tuning_content_params.FineTuningContentParams,
                ),
            ),
            cast_to=BinaryAPIResponse,
        )

    def estimate_price(
        self,
        *,
        training_file: str,
        from_checkpoint: str | Omit = omit,
        model: str | Omit = omit,
        n_epochs: int | Omit = omit,
        n_evals: int | Omit = omit,
        training_method: fine_tuning_estimate_price_params.TrainingMethod | Omit = omit,
        training_type: fine_tuning_estimate_price_params.TrainingType | Omit = omit,
        validation_file: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FineTuningEstimatePriceResponse:
        """
        Estimate the price of a fine-tuning job.

        Args:
          training_file: File-ID of a training file uploaded to the Together API

          from_checkpoint: The checkpoint identifier to continue training from a previous fine-tuning job.
              Format is `{$JOB_ID}` or `{$OUTPUT_MODEL_NAME}` or `{$JOB_ID}:{$STEP}` or
              `{$OUTPUT_MODEL_NAME}:{$STEP}`. The step value is optional; without it, the
              final checkpoint will be used.

          model: Name of the base model to run fine-tune job on

          n_epochs: Number of complete passes through the training dataset (higher values may
              improve results but increase cost and risk of overfitting)

          n_evals: Number of evaluations to be run on a given validation set during training

          training_method: The training method to use. 'sft' for Supervised Fine-Tuning or 'dpo' for Direct
              Preference Optimization.

          validation_file: File-ID of a validation file uploaded to the Together API

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/fine-tunes/estimate-price",
            body=maybe_transform(
                {
                    "training_file": training_file,
                    "from_checkpoint": from_checkpoint,
                    "model": model,
                    "n_epochs": n_epochs,
                    "n_evals": n_evals,
                    "training_method": training_method,
                    "training_type": training_type,
                    "validation_file": validation_file,
                },
                fine_tuning_estimate_price_params.FineTuningEstimatePriceParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FineTuningEstimatePriceResponse,
        )

    def list_checkpoints(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FineTuningListCheckpointsResponse:
        """
        List the checkpoints for a single fine-tuning job.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/fine-tunes/{id}/checkpoints",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FineTuningListCheckpointsResponse,
        )

    def list_events(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FineTuningListEventsResponse:
        """
        List the events for a single fine-tuning job.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/fine-tunes/{id}/events",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FineTuningListEventsResponse,
        )


class AsyncFineTuningResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncFineTuningResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/togethercomputer/together-py#accessing-raw-response-data-eg-headers
        """
        return AsyncFineTuningResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncFineTuningResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/togethercomputer/together-py#with_streaming_response
        """
        return AsyncFineTuningResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        training_file: str,
        model: str | None = None,
        n_epochs: int = 1,
        validation_file: str | None = "",
        n_evals: int | None = 0,
        n_checkpoints: int | None = 1,
        batch_size: int | Literal["max"] = "max",
        learning_rate: float | None = 0.00001,
        lr_scheduler_type: Literal["linear", "cosine"] = "cosine",
        min_lr_ratio: float = 0.0,
        scheduler_num_cycles: float = 0.5,
        warmup_ratio: float = 0.0,
        max_grad_norm: float = 1.0,
        weight_decay: float = 0.0,
        lora: bool = True,
        lora_r: int | None = None,
        lora_dropout: float | None = 0,
        lora_alpha: float | None = None,
        lora_trainable_modules: str | None = "all-linear",
        suffix: str | None = None,
        wandb_api_key: str | None = None,
        wandb_base_url: str | None = None,
        wandb_project_name: str | None = None,
        wandb_name: str | None = None,
        verbose: bool = False,
        model_limits: FinetuneTrainingLimits | None = None,
        train_on_inputs: bool | Literal["auto"] | None = None,
        training_method: str = "sft",
        dpo_beta: float | None = None,
        dpo_normalize_logratios_by_length: bool = False,
        rpo_alpha: float | None = None,
        simpo_gamma: float | None = None,
        from_checkpoint: str | None = None,
        from_hf_model: str | None = None,
        hf_model_revision: str | None = None,
        hf_api_token: str | None = None,
        hf_output_repo_name: str | None = None,
    ) -> FinetuneResponse:
        """
        Method to initiate a fine-tuning job

        Args:
            training_file (str): File-ID of a file uploaded to the Together API
            model (str, optional): Name of the base model to run fine-tune job on
            n_epochs (int, optional): Number of epochs for fine-tuning. Defaults to 1.
            validation file (str, optional): File ID of a file uploaded to the Together API for validation.
            n_evals (int, optional): Number of evaluation loops to run. Defaults to 0.
            n_checkpoints (int, optional): Number of checkpoints to save during fine-tuning.
                Defaults to 1.
            batch_size (int or "max"): Batch size for fine-tuning. Defaults to max.
            learning_rate (float, optional): Learning rate multiplier to use for training
                Defaults to 0.00001.
            lr_scheduler_type (Literal["linear", "cosine"]): Learning rate scheduler type. Defaults to "cosine".
            min_lr_ratio (float, optional): Min learning rate ratio of the initial learning rate for
                the learning rate scheduler. Defaults to 0.0.
            scheduler_num_cycles (float, optional): Number or fraction of cycles for the cosine learning rate scheduler. Defaults to 0.5.
            warmup_ratio (float, optional): Warmup ratio for the learning rate scheduler.
            max_grad_norm (float, optional): Max gradient norm. Defaults to 1.0, set to 0 to disable.
            weight_decay (float, optional): Weight decay. Defaults to 0.0.
            lora (bool, optional): Whether to use LoRA adapters. Defaults to True.
            lora_r (int, optional): Rank of LoRA adapters. Defaults to 8.
            lora_dropout (float, optional): Dropout rate for LoRA adapters. Defaults to 0.
            lora_alpha (float, optional): Alpha for LoRA adapters. Defaults to 8.
            lora_trainable_modules (str, optional): Trainable modules for LoRA adapters. Defaults to "all-linear".
            suffix (str, optional): Up to 40 character suffix that will be added to your fine-tuned model name.
                Defaults to None.
            wandb_api_key (str, optional): API key for Weights & Biases integration.
                Defaults to None.
            wandb_base_url (str, optional): Base URL for Weights & Biases integration.
                Defaults to None.
            wandb_project_name (str, optional): Project name for Weights & Biases integration.
                Defaults to None.
            wandb_name (str, optional): Run name for Weights & Biases integration.
                Defaults to None.
            verbose (bool, optional): whether to print the job parameters before submitting a request.
                Defaults to False.
            model_limits (FinetuneTrainingLimits, optional): Limits for the hyperparameters the model in Fine-tuning.
                Defaults to None.
            train_on_inputs (bool or "auto", optional): Whether to mask the user messages in conversational data or prompts in instruction data.
                "auto" will automatically determine whether to mask the inputs based on the data format.
                For datasets with the "text" field (general format), inputs will not be masked.
                For datasets with the "messages" field (conversational format) or "prompt" and "completion" fields
                (Instruction format), inputs will be masked.
                Defaults to None, or "auto" if training_method is "sft" (set in create_finetune_request).
            training_method (str, optional): Training method. Defaults to "sft".
                Supported methods: "sft", "dpo".
            dpo_beta (float, optional): DPO beta parameter. Defaults to None.
            dpo_normalize_logratios_by_length (bool): Whether or not normalize logratios by sample length. Defaults to False,
            rpo_alpha (float, optional): RPO alpha parameter of DPO training to include NLL in the loss. Defaults to None.
            simpo_gamma: (float, optional): SimPO gamma parameter. Defaults to None.
            from_checkpoint (str, optional): The checkpoint identifier to continue training from a previous fine-tuning job.
                The format: {$JOB_ID/$OUTPUT_MODEL_NAME}:{$STEP}.
                The step value is optional, without it the final checkpoint will be used.
            from_hf_model (str, optional): The Hugging Face Hub repo to start training from.
                Should be as close as possible to the base model (specified by the `model` argument) in terms of architecture and size.
            hf_model_revision (str, optional): The revision of the Hugging Face Hub model to continue training from. Defaults to None.
                Example: hf_model_revision=None (defaults to the latest revision in `main`) or
                hf_model_revision="607a30d783dfa663caf39e06633721c8d4cfcd7e" (specific commit).
            hf_api_token (str, optional): API key for the Hugging Face Hub. Defaults to None.
            hf_output_repo_name (str, optional): HF repo to upload the fine-tuned model to. Defaults to None.

        Returns:
            FinetuneResponse: Object containing information about fine-tuning job.
        """

        if model_limits is None:
            model_name = None
            # mypy doesn't understand that model or from_checkpoint is not None
            if model is not None:
                model_name = model
            elif from_checkpoint is not None:
                model_name = from_checkpoint.split(":")[0]
            else:
                # this branch is unreachable, but mypy doesn't know that
                pass
            model_limits = await async_get_model_limits(self._client, str(model_name))

        finetune_request, training_type_cls, training_method_cls = create_finetune_request(
            model_limits=model_limits,
            training_file=training_file,
            model=model,
            n_epochs=n_epochs,
            validation_file=validation_file,
            n_evals=n_evals,
            n_checkpoints=n_checkpoints,
            batch_size=batch_size,
            learning_rate=learning_rate,
            lr_scheduler_type=lr_scheduler_type,
            min_lr_ratio=min_lr_ratio,
            scheduler_num_cycles=scheduler_num_cycles,
            warmup_ratio=warmup_ratio,
            max_grad_norm=max_grad_norm,
            weight_decay=weight_decay,
            lora=lora,
            lora_r=lora_r,
            lora_dropout=lora_dropout,
            lora_alpha=lora_alpha,
            lora_trainable_modules=lora_trainable_modules,
            suffix=suffix,
            wandb_api_key=wandb_api_key,
            wandb_base_url=wandb_base_url,
            wandb_project_name=wandb_project_name,
            wandb_name=wandb_name,
            train_on_inputs=train_on_inputs,
            training_method=training_method,
            dpo_beta=dpo_beta,
            dpo_normalize_logratios_by_length=dpo_normalize_logratios_by_length,
            rpo_alpha=rpo_alpha,
            simpo_gamma=simpo_gamma,
            from_checkpoint=from_checkpoint,
            from_hf_model=from_hf_model,
            hf_model_revision=hf_model_revision,
            hf_api_token=hf_api_token,
            hf_output_repo_name=hf_output_repo_name,
        )


        price_estimation_result = await self.estimate_price(
            training_file=training_file,
            from_checkpoint=from_checkpoint or Omit(),
            validation_file=validation_file or Omit(),
            model=model or "",
            n_epochs=finetune_request.n_epochs,
            n_evals=finetune_request.n_evals or 0,
            training_type=training_type_cls,
            training_method=training_method_cls,
        )


        if verbose:
            rprint(
                "Submitting a fine-tuning job with the following parameters:",
                finetune_request,
            )
            if not price_estimation_result.allowed_to_proceed:
                rprint(
                    "[red]"
                    + _WARNING_MESSAGE_INSUFFICIENT_FUNDS.format(
                        price_estimation_result.estimated_total_price # pyright: ignore[reportPossiblyUnboundVariable]
                    )
                    + "[/red]",
                )
        parameter_payload = finetune_request.model_dump(exclude_none=True)

        return await self._client.post(
            "/fine-tunes",
            body=parameter_payload,
            cast_to=FinetuneResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FinetuneResponse:
        """
        List the metadata for a single fine-tuning job.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/fine-tunes/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FinetuneResponse,
        )

    async def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FineTuningListResponse:
        """List the metadata for all fine-tuning jobs.

        Returns a list of
        FinetuneResponseTruncated objects.
        """
        return await self._get(
            "/fine-tunes",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FineTuningListResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        force: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FineTuningDeleteResponse:
        """
        Delete a fine-tuning job.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/fine-tunes/{id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"force": force}, fine_tuning_delete_params.FineTuningDeleteParams),
            ),
            cast_to=FineTuningDeleteResponse,
        )

    async def cancel(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FineTuningCancelResponse:
        """Cancel a currently running fine-tuning job.

        Returns a FinetuneResponseTruncated
        object.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/fine-tunes/{id}/cancel",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FineTuningCancelResponse,
        )

    async def content(
        self,
        *,
        ft_id: str,
        checkpoint: Literal["merged", "adapter", "model_output_path"] | Omit = omit,
        checkpoint_step: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncBinaryAPIResponse:
        """
        Receive a compressed fine-tuned model or checkpoint.

        Args:
          ft_id: Fine-tune ID to download. A string that starts with `ft-`.

          checkpoint: Specifies checkpoint type to download - `merged` vs `adapter`. This field is
              required if the checkpoint_step is not set.

          checkpoint_step: Specifies step number for checkpoint to download. Ignores `checkpoint` value if
              set.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "application/octet-stream", **(extra_headers or {})}
        return await self._get(
            "/finetune/download",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "ft_id": ft_id,
                        "checkpoint": checkpoint,
                        "checkpoint_step": checkpoint_step,
                    },
                    fine_tuning_content_params.FineTuningContentParams,
                ),
            ),
            cast_to=AsyncBinaryAPIResponse,
        )

    async def estimate_price(
        self,
        *,
        training_file: str,
        from_checkpoint: str | Omit = omit,
        model: str | Omit = omit,
        n_epochs: int | Omit = omit,
        n_evals: int | Omit = omit,
        training_method: fine_tuning_estimate_price_params.TrainingMethod | Omit = omit,
        training_type: fine_tuning_estimate_price_params.TrainingType | Omit = omit,
        validation_file: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FineTuningEstimatePriceResponse:
        """
        Estimate the price of a fine-tuning job.

        Args:
          training_file: File-ID of a training file uploaded to the Together API

          from_checkpoint: The checkpoint identifier to continue training from a previous fine-tuning job.
              Format is `{$JOB_ID}` or `{$OUTPUT_MODEL_NAME}` or `{$JOB_ID}:{$STEP}` or
              `{$OUTPUT_MODEL_NAME}:{$STEP}`. The step value is optional; without it, the
              final checkpoint will be used.

          model: Name of the base model to run fine-tune job on

          n_epochs: Number of complete passes through the training dataset (higher values may
              improve results but increase cost and risk of overfitting)

          n_evals: Number of evaluations to be run on a given validation set during training

          training_method: The training method to use. 'sft' for Supervised Fine-Tuning or 'dpo' for Direct
              Preference Optimization.

          validation_file: File-ID of a validation file uploaded to the Together API

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/fine-tunes/estimate-price",
            body=await async_maybe_transform(
                {
                    "training_file": training_file,
                    "from_checkpoint": from_checkpoint,
                    "model": model,
                    "n_epochs": n_epochs,
                    "n_evals": n_evals,
                    "training_method": training_method,
                    "training_type": training_type,
                    "validation_file": validation_file,
                },
                fine_tuning_estimate_price_params.FineTuningEstimatePriceParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FineTuningEstimatePriceResponse,
        )

    async def list_checkpoints(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FineTuningListCheckpointsResponse:
        """
        List the checkpoints for a single fine-tuning job.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/fine-tunes/{id}/checkpoints",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FineTuningListCheckpointsResponse,
        )

    async def list_events(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FineTuningListEventsResponse:
        """
        List the events for a single fine-tuning job.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/fine-tunes/{id}/events",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FineTuningListEventsResponse,
        )


class FineTuningResourceWithRawResponse:
    def __init__(self, fine_tuning: FineTuningResource) -> None:
        self._fine_tuning = fine_tuning

        self.retrieve = to_raw_response_wrapper(
            fine_tuning.retrieve,
        )
        self.list = to_raw_response_wrapper(
            fine_tuning.list,
        )
        self.delete = to_raw_response_wrapper(
            fine_tuning.delete,
        )
        self.cancel = to_raw_response_wrapper(
            fine_tuning.cancel,
        )
        self.content = to_custom_raw_response_wrapper(
            fine_tuning.content,
            BinaryAPIResponse,
        )
        self.estimate_price = to_raw_response_wrapper(
            fine_tuning.estimate_price,
        )
        self.list_checkpoints = to_raw_response_wrapper(
            fine_tuning.list_checkpoints,
        )
        self.list_events = to_raw_response_wrapper(
            fine_tuning.list_events,
        )


class AsyncFineTuningResourceWithRawResponse:
    def __init__(self, fine_tuning: AsyncFineTuningResource) -> None:
        self._fine_tuning = fine_tuning

        self.retrieve = async_to_raw_response_wrapper(
            fine_tuning.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            fine_tuning.list,
        )
        self.delete = async_to_raw_response_wrapper(
            fine_tuning.delete,
        )
        self.cancel = async_to_raw_response_wrapper(
            fine_tuning.cancel,
        )
        self.content = async_to_custom_raw_response_wrapper(
            fine_tuning.content,
            AsyncBinaryAPIResponse,
        )
        self.estimate_price = async_to_raw_response_wrapper(
            fine_tuning.estimate_price,
        )
        self.list_checkpoints = async_to_raw_response_wrapper(
            fine_tuning.list_checkpoints,
        )
        self.list_events = async_to_raw_response_wrapper(
            fine_tuning.list_events,
        )


class FineTuningResourceWithStreamingResponse:
    def __init__(self, fine_tuning: FineTuningResource) -> None:
        self._fine_tuning = fine_tuning

        self.retrieve = to_streamed_response_wrapper(
            fine_tuning.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            fine_tuning.list,
        )
        self.delete = to_streamed_response_wrapper(
            fine_tuning.delete,
        )
        self.cancel = to_streamed_response_wrapper(
            fine_tuning.cancel,
        )
        self.content = to_custom_streamed_response_wrapper(
            fine_tuning.content,
            StreamedBinaryAPIResponse,
        )
        self.estimate_price = to_streamed_response_wrapper(
            fine_tuning.estimate_price,
        )
        self.list_checkpoints = to_streamed_response_wrapper(
            fine_tuning.list_checkpoints,
        )
        self.list_events = to_streamed_response_wrapper(
            fine_tuning.list_events,
        )


class AsyncFineTuningResourceWithStreamingResponse:
    def __init__(self, fine_tuning: AsyncFineTuningResource) -> None:
        self._fine_tuning = fine_tuning

        self.retrieve = async_to_streamed_response_wrapper(
            fine_tuning.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            fine_tuning.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            fine_tuning.delete,
        )
        self.cancel = async_to_streamed_response_wrapper(
            fine_tuning.cancel,
        )
        self.content = async_to_custom_streamed_response_wrapper(
            fine_tuning.content,
            AsyncStreamedBinaryAPIResponse,
        )
        self.estimate_price = async_to_streamed_response_wrapper(
            fine_tuning.estimate_price,
        )
        self.list_checkpoints = async_to_streamed_response_wrapper(
            fine_tuning.list_checkpoints,
        )
        self.list_events = async_to_streamed_response_wrapper(
            fine_tuning.list_events,
        )



from __future__ import annotations

import os
import logging
from datetime import datetime

logger = logging.getLogger("together")

TOGETHER_LOG = os.environ.get("TOGETHER_LOG")

NANODOLLAR = 1_000_000_000


def enforce_trailing_slash(url: str) -> str:
    if not url.endswith("/"):
        return url + "/"
    else:
        return url


def normalize_key(key: str) -> str:
    return key.replace("/", "--").replace("_", "-").replace(" ", "-").lower()


def parse_timestamp(timestamp: str) -> datetime:
    formats = ["%Y-%m-%dT%H:%M:%S.%fZ", "%Y-%m-%dT%H:%M:%SZ"]
    for fmt in formats:
        try:
            return datetime.strptime(timestamp, fmt)
        except ValueError:
            continue
    raise ValueError("Timestamp does not match any expected format")


# Convert fine-tune nano-dollar price to dollars
def finetune_price_to_dollars(price: float) -> float:
    return price / NANODOLLAR


def convert_bytes(num: float) -> str | None:
    """
    Convert bytes to a human-readable format.

    Args:
        num (int): Number of bytes.

    Returns:
        str: Human-readable representation of the size.
    """
    for unit in ["B", "KB", "MB", "GB", "TB", "PB"]:
        if num < 1024.0:
            return "{:.1f} {}".format(num, unit)
        num /= 1024.0

    return None


def convert_unix_timestamp(timestamp: int) -> str:
    """
    Convert a Unix timestamp to a human-readable date and time format.

    Args:
        timestamp (int): Unix timestamp.

    Returns:
        str: Human-readable date and time string.
    """
    # Convert Unix timestamp to datetime object
    dt_object = datetime.fromtimestamp(timestamp)

    # Format datetime object as ISO 8601 string
    iso_format = dt_object.strftime("%Y-%m-%dT%H:%M:%S.%fZ")

    return iso_format


def format_timestamp(timestamp_str: str) -> str:
    """Format timestamp to a readable date string.

    Args:
        timestamp: A timestamp string

    Returns:
        str: Formatted timestamp string (MM/DD/YYYY, HH:MM AM/PM)
    """
    try:
        timestamp = parse_timestamp(timestamp_str)
        return timestamp.strftime("%m/%d/%Y, %I:%M %p")
    except ValueError:
        return ""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["FineTuningEstimatePriceResponse"]


class FineTuningEstimatePriceResponse(BaseModel):
    allowed_to_proceed: Optional[bool] = None
    """Whether the user is allowed to proceed with the fine-tuning job"""

    estimated_eval_token_count: Optional[float] = None
    """The estimated number of tokens for evaluation"""

    estimated_total_price: Optional[float] = None
    """The price of the fine-tuning job"""

    estimated_train_token_count: Optional[float] = None
    """The estimated number of tokens to be trained"""

    user_limit: Optional[float] = None
    """The user's credit limit in dollars"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from ..._models import BaseModel
from ..tool_choice import ToolChoice
from .chat_completion_usage import ChatCompletionUsage
from .chat_completion_warning import ChatCompletionWarning

__all__ = ["ChatCompletionChunk", "Choice", "ChoiceDelta", "ChoiceDeltaFunctionCall"]


class ChoiceDeltaFunctionCall(BaseModel):
    arguments: str

    name: str


class ChoiceDelta(BaseModel):
    role: Literal["system", "user", "assistant", "function", "tool"]

    content: Optional[str] = None

    function_call: Optional[ChoiceDeltaFunctionCall] = None

    reasoning: Optional[str] = None

    token_id: Optional[int] = None

    tool_calls: Optional[List[ToolChoice]] = None


class Choice(BaseModel):
    delta: ChoiceDelta

    finish_reason: Optional[Literal["stop", "eos", "length", "tool_calls", "function_call"]] = None

    index: int

    logprobs: Optional[float] = None

    seed: Optional[int] = None


class ChatCompletionChunk(BaseModel):
    id: str

    choices: List[Choice]

    created: int

    model: str

    object: Literal["chat.completion.chunk"]

    system_fingerprint: Optional[str] = None

    usage: Optional[ChatCompletionUsage] = None

    warnings: Optional[List[ChatCompletionWarning]] = None
